# Copyright: (c) 2019, Olivier Blin <olivier.oblin@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
from ansible_collections.cisco.nxos.plugins.modules import nxos_l3_interface
from .nxos_module import TestNxosModule, load_fixture, set_module_args


class TestNxosL3InterfaceModule(TestNxosModule):

    module = nxos_l3_interface

    def setUp(self):
        super(TestNxosL3InterfaceModule, self).setUp()

        self.mock_load_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_l3_interface.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_l3_interface.get_config"
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestNxosL3InterfaceModule, self).tearDown()
        self.mock_load_config.stop()
        self.mock_get_config.stop()

    def load_fixtures(self, commands=None, device=""):
        self.load_config.return_value = None
        self.get_config.return_value = load_fixture(
            "nxos_l3_interface", self.mode
        )

    def test_nxos_l3_interface_unknonw_ethernet(self):
        self.mode = "ethernet_noshut"
        set_module_args(dict(name="Ethernet1/2", ipv4="192.168.0.1/24"))
        result = self.execute_module(changed=False)

    # Add when missing
    def test_nxos_l3_interface_add_missing_ipv4(self):
        self.mode = "ethernet_noshut"
        set_module_args(dict(name="Ethernet1/1", ipv4="192.168.0.1/24"))
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["interface Ethernet1/1", "ip address 192.168.0.1/24", "exit"],
        )

    def test_nxos_l3_interface_add_missing_ipv4_on_e11(self):
        self.mode = "ethernet_noshut"
        set_module_args(dict(name="et1/1", ipv4="192.168.0.1/24"))
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["interface Ethernet1/1", "ip address 192.168.0.1/24", "exit"],
        )

    def test_nxos_l3_interface_add_missing_ipv6(self):
        self.mode = "ethernet_noshut"
        set_module_args(dict(name="Ethernet1/1", ipv6="2001:db8::1/124"))
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["interface Ethernet1/1", "ipv6 address 2001:db8::1/124", "exit"],
        )

    def test_nxos_l3_interface_add_missing_ipv4_and_ipv6(self):
        self.mode = "ethernet_noshut"
        set_module_args(
            dict(
                name="Ethernet1/1",
                ipv4="192.168.0.1/24",
                ipv6="2001:db8::1/124",
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "ip address 192.168.0.1/24",
                "ipv6 address 2001:db8::1/124",
                "exit",
            ],
        )

    # Add when existing
    def test_nxos_l3_interface_add_existing_ipv4(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(dict(name="Ethernet1/1", ipv4="192.168.0.1/24"))
        result = self.execute_module()

    def test_nxos_l3_interface_add_existing_ipv4_on_e11(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(dict(name="et1/1", ipv4="192.168.0.1/24"))
        result = self.execute_module()

    def test_nxos_l3_interface_add_existing_ipv6(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(dict(name="Ethernet1/1", ipv6="2001:db8::1/124"))
        result = self.execute_module()

    def test_nxos_l3_interface_add_existing_ipv4_and_ipv6(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(
            dict(
                name="Ethernet1/1",
                ipv4="192.168.0.1/24",
                ipv6="2001:db8::1/124",
            )
        )
        result = self.execute_module()

    def test_nxos_l3_interface_new_ipv4_and_ipv6(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(
            dict(
                name="Ethernet1/1",
                ipv4="192.168.0.2/24",
                ipv6="2001:db8::2/124",
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "ip address 192.168.0.2/24",
                "ipv6 address 2001:db8::2/124",
                "exit",
            ],
        )

    # Add when existing with multiple IPv6
    def test_nxos_l3_interface_multiple_ipv6_add_first(self):
        self.mode = "ethernet_noshut_multiple_ipv6"
        set_module_args(dict(name="Ethernet1/1", ipv6="2001:db8::1/124"))
        result = self.execute_module()

    def test_nxos_l3_interface_multiple_ipv6_add_last(self):
        self.mode = "ethernet_noshut_multiple_ipv6"
        set_module_args(dict(name="Ethernet1/1", ipv6="2001:db8:2::1/124"))
        result = self.execute_module()

    # Add aggregate
    def test_nxos_l3_interface_add_missing_with_empty_aggregate(self):
        self.mode = "ethernet_noshut"
        set_module_args(dict(aggregate=[]))
        result = self.execute_module()

    def test_nxos_l3_interface_add_missing_with_aggregate(self):
        self.mode = "ethernet_noshut"
        set_module_args(
            dict(
                aggregate=[
                    dict(
                        name="Ethernet1/1",
                        ipv4="192.168.0.2/24",
                        ipv6="2001:db8::2/124",
                    ),
                    dict(name="Ethernet1/1", ipv6="2001:db8:1::2/124"),
                    dict(name="Ethernet1/1", ipv6="2001:db8:2::2/124"),
                ]
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "ip address 192.168.0.2/24",
                "ipv6 address 2001:db8::2/124",
                "exit",
                "interface Ethernet1/1",
                "ipv6 address 2001:db8:1::2/124",
                "exit",
                "interface Ethernet1/1",
                "ipv6 address 2001:db8:2::2/124",
                "exit",
            ],
        )

    # Rem when missing
    def test_nxos_l3_interface_rem_missing_ipv4(self):
        self.mode = "ethernet_noshut"
        set_module_args(
            dict(name="Ethernet1/1", ipv4="192.168.0.1/24", state="absent")
        )
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_ipv4_on_e11(self):
        self.mode = "ethernet_noshut"
        set_module_args(
            dict(name="et1/1", ipv4="192.168.0.1/24", state="absent")
        )
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_ipv6(self):
        self.mode = "ethernet_noshut"
        set_module_args(
            dict(name="Ethernet1/1", ipv6="2001:db8::1/124", state="absent")
        )
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_ipv4_and_ipv6(self):
        self.mode = "ethernet_noshut"
        set_module_args(
            dict(
                name="Ethernet1/1",
                ipv4="192.168.0.1/24",
                ipv6="2001:db8::1/124",
                state="absent",
            )
        )
        result = self.execute_module()

    # Rem when existing
    def test_nxos_l3_interface_rem_existing_ipv4(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(
            dict(name="Ethernet1/1", ipv4="192.168.0.1/24", state="absent")
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["interface Ethernet1/1", "no ip address 192.168.0.1/24", "exit"],
        )

    def test_nxos_l3_interface_rem_existing_ipv4_on_e11(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(
            dict(name="et1/1", ipv4="192.168.0.1/24", state="absent")
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["interface Ethernet1/1", "no ip address 192.168.0.1/24", "exit"],
        )

    def test_nxos_l3_interface_rem_existing_ipv6(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(
            dict(name="Ethernet1/1", ipv6="2001:db8::1/124", state="absent")
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "no ipv6 address 2001:db8::1/124",
                "exit",
            ],
        )

    def test_nxos_l3_interface_rem_existing_ipv4_and_ipv6(self):
        self.mode = "ethernet_noshut_ipv4_ipv6"
        set_module_args(
            dict(
                name="Ethernet1/1",
                ipv4="192.168.0.1/24",
                ipv6="2001:db8::1/124",
                state="absent",
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "no ip address 192.168.0.1/24",
                "no ipv6 address 2001:db8::1/124",
                "exit",
            ],
        )

    # Rem when existing with multiple IPv6
    def test_nxos_l3_interface_multiple_ipv6_rem_first(self):
        self.mode = "ethernet_noshut_multiple_ipv6"
        set_module_args(
            dict(name="Ethernet1/1", ipv6="2001:db8::1/124", state="absent")
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "no ipv6 address 2001:db8::1/124",
                "exit",
            ],
        )

    def test_nxos_l3_interface_multiple_ipv6_rem_last(self):
        self.mode = "ethernet_noshut_multiple_ipv6"
        set_module_args(
            dict(name="Ethernet1/1", ipv6="2001:db8:2::1/124", state="absent")
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "no ipv6 address 2001:db8:2::1/124",
                "exit",
            ],
        )

    # Rem when missing with aggregate
    def test_nxos_l3_interface_rem_with_empty_aggregate(self):
        self.mode = "ethernet_noshut_multiple_ipv6"
        set_module_args(dict(aggregate=[], state="absent"))
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_with_aggregate(self):
        self.mode = "ethernet_noshut_multiple_ipv6"
        set_module_args(
            dict(
                state="absent",
                aggregate=[
                    dict(
                        name="Ethernet1/1",
                        ipv4="192.168.0.2/24",
                        ipv6="2001:db8::2/124",
                    ),
                    dict(name="Ethernet1/1", ipv6="2001:db8:1::2/124"),
                    dict(name="Ethernet1/1", ipv6="2001:db8:2::2/124"),
                ],
            )
        )
        result = self.execute_module()

    # Rem when existing with aggregate
    def test_nxos_l3_interface_rem_existing_with_aggregate(self):
        self.mode = "ethernet_noshut_multiple_ipv6"
        set_module_args(
            dict(
                state="absent",
                aggregate=[
                    dict(
                        name="Ethernet1/1",
                        ipv4="192.168.0.1/24",
                        ipv6="2001:db8::1/124",
                    ),
                    dict(name="Ethernet1/1", ipv6="2001:db8:1::1/124"),
                    dict(name="Ethernet1/1", ipv6="2001:db8:2::1/124"),
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "interface Ethernet1/1",
                "no ip address 192.168.0.1/24",
                "no ipv6 address 2001:db8::1/124",
                "exit",
                "interface Ethernet1/1",
                "no ipv6 address 2001:db8:1::1/124",
                "exit",
                "interface Ethernet1/1",
                "no ipv6 address 2001:db8:2::1/124",
                "exit",
            ],
        )

    # Add itf only
    def test_nxos_l3_interface_add_on_itf_only(self):
        self.mode = "ethernet_noshut"
        set_module_args(dict(name="Ethernet1/1"))
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], ["interface Ethernet1/1"])

    # Add unknown interface
    def test_nxos_l3_interface_add_on_unknown_itf(self):
        self.mode = "ethernet_noshut"
        set_module_args(dict(name="Ethernet1/2", ipv4="192.168.0.1/24"))
        result = self.execute_module()
        self.assertEqual(result["warnings"], ["Unknown interface Ethernet1/2"])

    # Rem unknown interface
    def test_nxos_l3_interface_rem_on_unknown_itf(self):
        self.mode = "ethernet_noshut"
        set_module_args(
            dict(name="Ethernet1/2", ipv4="192.168.0.1/24", state="absent")
        )
        result = self.execute_module()
        self.assertEqual(result["warnings"], ["Unknown interface Ethernet1/2"])



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Andrew Klychkov (@Andersson007) <aaklychkov@mail.ru>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: postgresql_user_obj_stat_info
short_description: Gather statistics about PostgreSQL user objects
description:
- Gathers statistics about PostgreSQL user objects.
version_added: '0.2.0'
options:
  filter:
    description:
    - Limit the collected information by comma separated string or YAML list.
    - Allowable values are C(functions), C(indexes), C(tables).
    - By default, collects all subsets.
    - Unsupported values are ignored.
    type: list
    elements: str
  schema:
    description:
    - Restrict the output by certain schema.
    type: str
  db:
    description:
    - Name of database to connect.
    type: str
    aliases:
    - login_db
  session_role:
    description:
    - Switch to session_role after connecting. The specified session_role must
      be a role that the current login_user is a member of.
    - Permissions checking for SQL commands is carried out as though
      the session_role were the one that had logged in originally.
    type: str
  trust_input:
    description:
    - If C(no), check the value of I(session_role) is potentially dangerous.
    - It makes sense to use C(no) only when SQL injections via I(session_role) are possible.
    type: bool
    default: yes
    version_added: '0.2.0'

notes:
- C(size) and C(total_size) returned values are presented in bytes.
- For tracking function statistics the PostgreSQL C(track_functions) parameter must be enabled.
  See U(https://www.postgresql.org/docs/current/runtime-config-statistics.html) for more information.
seealso:
- module: community.general.postgresql_info
- module: community.general.postgresql_ping
- name: PostgreSQL statistics collector reference
  description: Complete reference of the PostgreSQL statistics collector documentation.
  link: https://www.postgresql.org/docs/current/monitoring-stats.html
author:
- Andrew Klychkov (@Andersson007)
- Thomas O'Donnell (@andytom)
extends_documentation_fragment:
- community.general.postgres

'''

EXAMPLES = r'''
- name: Collect information about all supported user objects of the acme database
  community.general.postgresql_user_obj_stat_info:
    db: acme

- name: Collect information about all supported user objects in the custom schema of the acme database
  community.general.postgresql_user_obj_stat_info:
    db: acme
    schema: custom

- name: Collect information about user tables and indexes in the acme database
  community.general.postgresql_user_obj_stat_info:
    db: acme
    filter: tables, indexes
'''

RETURN = r'''
indexes:
  description: User index statistics
  returned: always
  type: dict
  sample: {"public": {"test_id_idx": {"idx_scan": 0, "idx_tup_fetch": 0, "idx_tup_read": 0, "relname": "test", "size": 8192, ...}}}
tables:
  description: User table statistics.
  returned: always
  type: dict
  sample: {"public": {"test": {"analyze_count": 3, "n_dead_tup": 0, "n_live_tup": 0, "seq_scan": 2, "size": 0, "total_size": 8192, ...}}}
functions:
  description: User function statistics.
  returned: always
  type: dict
  sample: {"public": {"inc": {"calls": 1, "funcid": 26722, "self_time": 0.23, "total_time": 0.23}}}
'''

try:
    from psycopg2.extras import DictCursor
except ImportError:
    # psycopg2 is checked by connect_to_db()
    # from ansible.module_utils.postgres
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.database import (
    check_input,
)
from ansible_collections.community.general.plugins.module_utils.postgres import (
    connect_to_db,
    exec_sql,
    get_conn_params,
    postgres_common_argument_spec,
)
from ansible.module_utils.six import iteritems


# ===========================================
# PostgreSQL module specific support methods.
#


class PgUserObjStatInfo():
    """Class to collect information about PostgreSQL user objects.

    Args:
        module (AnsibleModule): Object of AnsibleModule class.
        cursor (cursor): Cursor object of psycopg2 library to work with PostgreSQL.

    Attributes:
        module (AnsibleModule): Object of AnsibleModule class.
        cursor (cursor): Cursor object of psycopg2 library to work with PostgreSQL.
        executed_queries (list): List of executed queries.
        info (dict): Statistics dictionary.
        obj_func_mapping (dict): Mapping of object types to corresponding functions.
        schema (str): Name of a schema to restrict stat collecting.
    """

    def __init__(self, module, cursor):
        self.module = module
        self.cursor = cursor
        self.info = {
            'functions': {},
            'indexes': {},
            'tables': {},
        }
        self.obj_func_mapping = {
            'functions': self.get_func_stat,
            'indexes': self.get_idx_stat,
            'tables': self.get_tbl_stat,
        }
        self.schema = None

    def collect(self, filter_=None, schema=None):
        """Collect statistics information of user objects.

        Kwargs:
            filter_ (list): List of subsets which need to be collected.
            schema (str): Restrict stat collecting by certain schema.

        Returns:
            ``self.info``.
        """
        if schema:
            self.set_schema(schema)

        if filter_:
            for obj_type in filter_:
                obj_type = obj_type.strip()
                obj_func = self.obj_func_mapping.get(obj_type)

                if obj_func is not None:
                    obj_func()
                else:
                    self.module.warn("Unknown filter option '%s'" % obj_type)

        else:
            for obj_func in self.obj_func_mapping.values():
                obj_func()

        return self.info

    def get_func_stat(self):
        """Get function statistics and fill out self.info dictionary."""
        query = "SELECT * FROM pg_stat_user_functions"
        if self.schema:
            query = "SELECT * FROM pg_stat_user_functions WHERE schemaname = %s"

        result = exec_sql(self, query, query_params=(self.schema,),
                          add_to_executed=False)

        if not result:
            return

        self.__fill_out_info(result,
                             info_key='functions',
                             schema_key='schemaname',
                             name_key='funcname')

    def get_idx_stat(self):
        """Get index statistics and fill out self.info dictionary."""
        query = "SELECT * FROM pg_stat_user_indexes"
        if self.schema:
            query = "SELECT * FROM pg_stat_user_indexes WHERE schemaname = %s"

        result = exec_sql(self, query, query_params=(self.schema,),
                          add_to_executed=False)

        if not result:
            return

        self.__fill_out_info(result,
                             info_key='indexes',
                             schema_key='schemaname',
                             name_key='indexrelname')

    def get_tbl_stat(self):
        """Get table statistics and fill out self.info dictionary."""
        query = "SELECT * FROM pg_stat_user_tables"
        if self.schema:
            query = "SELECT * FROM pg_stat_user_tables WHERE schemaname = %s"

        result = exec_sql(self, query, query_params=(self.schema,),
                          add_to_executed=False)

        if not result:
            return

        self.__fill_out_info(result,
                             info_key='tables',
                             schema_key='schemaname',
                             name_key='relname')

    def __fill_out_info(self, result, info_key=None, schema_key=None, name_key=None):
        # Convert result to list of dicts to handle it easier:
        result = [dict(row) for row in result]

        for elem in result:
            # Add schema name as a key if not presented:
            if not self.info[info_key].get(elem[schema_key]):
                self.info[info_key][elem[schema_key]] = {}

            # Add object name key as a subkey
            # (they must be uniq over a schema, so no need additional checks):
            self.info[info_key][elem[schema_key]][elem[name_key]] = {}

            # Add other other attributes to a certain index:
            for key, val in iteritems(elem):
                if key not in (schema_key, name_key):
                    self.info[info_key][elem[schema_key]][elem[name_key]][key] = val

            if info_key in ('tables', 'indexes'):
                schemaname = elem[schema_key]
                if self.schema:
                    schemaname = self.schema

                relname = '%s.%s' % (schemaname, elem[name_key])

                result = exec_sql(self, "SELECT pg_relation_size (%s)",
                                  query_params=(relname,),
                                  add_to_executed=False)

                self.info[info_key][elem[schema_key]][elem[name_key]]['size'] = result[0][0]

                if info_key == 'tables':
                    result = exec_sql(self, "SELECT pg_total_relation_size (%s)",
                                      query_params=(relname,),
                                      add_to_executed=False)

                    self.info[info_key][elem[schema_key]][elem[name_key]]['total_size'] = result[0][0]

    def set_schema(self, schema):
        """If schema exists, sets self.schema, otherwise fails."""
        query = ("SELECT 1 FROM information_schema.schemata "
                 "WHERE schema_name = %s")
        result = exec_sql(self, query, query_params=(schema,),
                          add_to_executed=False)

        if result and result[0][0]:
            self.schema = schema
        else:
            self.module.fail_json(msg="Schema '%s' does not exist" % (schema))


# ===========================================
# Module execution.
#

def main():
    argument_spec = postgres_common_argument_spec()
    argument_spec.update(
        db=dict(type='str', aliases=['login_db']),
        filter=dict(type='list', elements='str'),
        session_role=dict(type='str'),
        schema=dict(type='str'),
        trust_input=dict(type="bool", default=True),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    filter_ = module.params["filter"]
    schema = module.params["schema"]

    if not module.params["trust_input"]:
        check_input(module, module.params['session_role'])

    # Connect to DB and make cursor object:
    pg_conn_params = get_conn_params(module, module.params)
    # We don't need to commit anything, so, set it to False:
    db_connection = connect_to_db(module, pg_conn_params, autocommit=False)
    cursor = db_connection.cursor(cursor_factory=DictCursor)

    ############################
    # Create object and do work:
    pg_obj_info = PgUserObjStatInfo(module, cursor)

    info_dict = pg_obj_info.collect(filter_, schema)

    # Clean up:
    cursor.close()
    db_connection.close()

    # Return information:
    module.exit_json(**info_dict)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2018, KubeVirt Team <@kubevirt>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):
    DOCUMENTATION = r'''
options:
    resource_definition:
      description:
      - "A partial YAML definition of the object being created/updated. Here you can define Kubernetes
        resource parameters not covered by this module's parameters."
      - "NOTE: I(resource_definition) has lower priority than module parameters. If you try to define e.g.
        I(metadata.namespace) here, that value will be ignored and I(namespace) used instead."
      aliases:
      - definition
      - inline
      type: dict
    wait:
        description:
            - "I(True) if the module should wait for the resource to get into desired state."
        type: bool
        default: yes
    force:
       description:
            - If set to C(no), and I(state) is C(present), an existing object will be replaced.
       type: bool
       default: no
    wait_timeout:
        description:
            - The amount of time in seconds the module should wait for the resource to get into desired state.
        type: int
        default: 120
    wait_sleep:
        description:
            - Number of seconds to sleep between checks.
        default: 5
    memory:
        description:
            - The amount of memory to be requested by virtual machine.
            - For example 1024Mi.
        type: str
    memory_limit:
        description:
            - The maximum memory to be used by virtual machine.
            - For example 1024Mi.
        type: str
    machine_type:
        description:
            - QEMU machine type is the actual chipset of the virtual machine.
        type: str
    merge_type:
        description:
            - Whether to override the default patch merge approach with a specific type.
            - If more than one merge type is given, the merge types will be tried in order.
            - "Defaults to C(['strategic-merge', 'merge']), which is ideal for using the same parameters
              on resource kinds that combine Custom Resources and built-in resources, as
              Custom Resource Definitions typically aren't updatable by the usual strategic merge."
            - "See U(https://kubernetes.io/docs/tasks/run-application/update-api-object-kubectl-patch/#use-a-json-merge-patch-to-update-a-deployment)"
        type: list
        choices: [ json, merge, strategic-merge ]
    cpu_shares:
        description:
            - "Specify CPU shares."
        type: int
    cpu_limit:
        description:
            - "Is converted to its millicore value and multiplied by 100. The resulting value is the total amount of CPU time that a container can use
               every 100ms. A virtual machine cannot use more than its share of CPU time during this interval."
        type: int
    cpu_cores:
        description:
            - "Number of CPU cores."
        type: int
    cpu_model:
        description:
            - "CPU model."
            - "You can check list of available models here: U(https://github.com/libvirt/libvirt/blob/master/src/cpu_map/index.xml)."
            - "I(Note:) User can define default CPU model via as I(default-cpu-model) in I(kubevirt-config) I(ConfigMap), if not set I(host-model) is used."
            - "I(Note:) Be sure that node CPU model where you run a VM, has the same or higher CPU family."
            - "I(Note:) If CPU model wasn't defined, the VM will have CPU model closest to one that used on the node where the VM is running."
        type: str
    bootloader:
        description:
            - "Specify the bootloader of the virtual machine."
            - "All virtual machines use BIOS by default for booting."
        type: str
    smbios_uuid:
        description:
            - "In order to provide a consistent view on the virtualized hardware for the guest OS, the SMBIOS UUID can be set."
        type: str
    cpu_features:
        description:
            - "List of dictionary to fine-tune features provided by the selected CPU model."
            - "I(Note): Policy attribute can either be omitted or contain one of the following policies: force, require, optional, disable, forbid."
            - "I(Note): In case a policy is omitted for a feature, it will default to require."
            - "More information about policies: U(https://libvirt.org/formatdomain.html#elementsCPU)"
        type: list
    headless:
        description:
            - "Specify if the virtual machine should have attached a  minimal Video and Graphics device configuration."
            - "By default a minimal Video and Graphics device configuration will be applied to the VirtualMachineInstance. The video device is vga
               compatible and comes with a memory size of 16 MB."
    hugepage_size:
        description:
            - "Specify huge page size."
        type: str
    tablets:
        description:
            - "Specify tablets to be used as input devices"
        type: list
    hostname:
        description:
            - "Specifies the hostname of the virtual machine. The hostname will be set either by dhcp, cloud-init if configured or virtual machine
               name will be used."
    subdomain:
        description:
            - "If specified, the fully qualified virtual machine hostname will be hostname.subdomain.namespace.svc.cluster_domain. If not specified,
               the virtual machine will not have a domain name at all. The DNS entry will resolve to the virtual machine, no matter if the virtual machine
               itself can pick up a hostname."
requirements:
    - python >= 2.7
    - openshift >= 0.8.2
notes:
  - "In order to use this module you have to install Openshift Python SDK.
     To ensure it's installed with correct version you can create the following task:
     I(pip: name=openshift>=0.8.2)"
'''



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: s3_metrics_configuration
version_added: 1.3.0
short_description: Manage s3 bucket metrics configuration in AWS
description:
    - Manage s3 bucket metrics configuration in AWS which allows to get the CloudWatch request metrics for the objects in a bucket
author: Dmytro Vorotyntsev (@vorotech)
notes:
    - This modules manages single metrics configuration, the s3 bucket might have up to 1,000 metrics configurations
    - To request metrics for the entire bucket, create a metrics configuration without a filter
    - Metrics configurations are necessary only to enable request metric, bucket-level daily storage metrics are always turned on
options:
  bucket_name:
    description:
      - "Name of the s3 bucket"
    required: true
    type: str
  id:
    description:
      - "The ID used to identify the metrics configuration"
    required: true
    type: str
  filter_prefix:
    description:
      - "A prefix used when evaluating a metrics filter"
    required: false
    type: str
  filter_tags:
    description:
      - "A dictionary of one or more tags used when evaluating a metrics filter"
    required: false
    aliases: ['filter_tag']
    type: dict
  state:
    description:
      - "Create or delete metrics configuration"
    default: present
    choices: ['present', 'absent']
    type: str
extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

'''

EXAMPLES = r'''
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create a metrics configuration that enables metrics for an entire bucket
  community.aws.s3_metrics_configuration:
    bucket_name: my-bucket
    id: EntireBucket
    state: present

- name: Put a metrics configuration that enables metrics for objects starting with a prefix
  community.aws.s3_metrics_configuration:
    bucket_name: my-bucket
    id: Assets
    filter_prefix: assets
    state: present

- name: Put a metrics configuration that enables metrics for objects with specific tag
  community.aws.s3_metrics_configuration:
    bucket_name: my-bucket
    id: Assets
    filter_tag:
      kind: asset
    state: present

- name: Put a metrics configuration that enables metrics for objects that start with a particular prefix and have specific tags applied
  community.aws.s3_metrics_configuration:
    bucket_name: my-bucket
    id: ImportantBlueDocuments
    filter_prefix: documents
    filter_tags:
      priority: high
      class: blue
    state: present

- name: Delete metrics configuration
  community.aws.s3_metrics_configuration:
    bucket_name: my-bucket
    id: EntireBucket
    state: absent

'''

try:
    import boto3
    import botocore
    from botocore.exceptions import ClientError, BotoCoreError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule, is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AWSRetry, ansible_dict_to_boto3_tag_list


def _create_metrics_configuration(mc_id, filter_prefix, filter_tags):
    payload = {
        'Id': mc_id
    }
    # Just a filter_prefix or just a single tag filter is a special case
    if filter_prefix and not filter_tags:
        payload['Filter'] = {
            'Prefix': filter_prefix
        }
    elif not filter_prefix and len(filter_tags) == 1:
        payload['Filter'] = {
            'Tag': ansible_dict_to_boto3_tag_list(filter_tags)[0]
        }
    # Otherwise we need to use 'And'
    elif filter_tags:
        payload['Filter'] = {
            'And': {
                'Tags': ansible_dict_to_boto3_tag_list(filter_tags)
            }
        }
        if filter_prefix:
            payload['Filter']['And']['Prefix'] = filter_prefix

    return payload


def create_or_update_metrics_configuration(client, module):
    bucket_name = module.params.get('bucket_name')
    mc_id = module.params.get('id')
    filter_prefix = module.params.get('filter_prefix')
    filter_tags = module.params.get('filter_tags')

    try:
        response = client.get_bucket_metrics_configuration(aws_retry=True, Bucket=bucket_name, Id=mc_id)
        metrics_configuration = response['MetricsConfiguration']
    except is_boto3_error_code('NoSuchConfiguration'):
        metrics_configuration = None
    except (BotoCoreError, ClientError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to get bucket metrics configuration")

    new_configuration = _create_metrics_configuration(mc_id, filter_prefix, filter_tags)

    if metrics_configuration:
        if metrics_configuration == new_configuration:
            module.exit_json(changed=False)

    if module.check_mode:
        module.exit_json(changed=True)

    try:
        client.put_bucket_metrics_configuration(
            aws_retry=True,
            Bucket=bucket_name,
            Id=mc_id,
            MetricsConfiguration=new_configuration
        )
    except (BotoCoreError, ClientError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to put bucket metrics configuration '%s'" % mc_id)

    module.exit_json(changed=True)


def delete_metrics_configuration(client, module):
    bucket_name = module.params.get('bucket_name')
    mc_id = module.params.get('id')

    try:
        client.get_bucket_metrics_configuration(aws_retry=True, Bucket=bucket_name, Id=mc_id)
    except is_boto3_error_code('NoSuchConfiguration'):
        module.exit_json(changed=False)
    except (BotoCoreError, ClientError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to get bucket metrics configuration")

    if module.check_mode:
        module.exit_json(changed=True)

    try:
        client.delete_bucket_metrics_configuration(aws_retry=True, Bucket=bucket_name, Id=mc_id)
    except is_boto3_error_code('NoSuchConfiguration'):
        module.exit_json(changed=False)
    except (BotoCoreError, ClientError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to delete bucket metrics configuration '%s'" % mc_id)

    module.exit_json(changed=True)


def main():
    argument_spec = dict(
        bucket_name=dict(type='str', required=True),
        id=dict(type='str', required=True),
        filter_prefix=dict(type='str', required=False),
        filter_tags=dict(default={}, type='dict', required=False, aliases=['filter_tag']),
        state=dict(default='present', type='str', choices=['present', 'absent']),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True
    )

    state = module.params.get('state')

    try:
        client = module.client('s3', retry_decorator=AWSRetry.exponential_backoff(retries=10, delay=3))
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg='Failed to connect to AWS')

    if state == 'present':
        create_or_update_metrics_configuration(client, module)
    elif state == 'absent':
        delete_metrics_configuration(client, module)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The asa_og fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type


from copy import deepcopy
from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import (
    utils,
)
from ansible_collections.cisco.asa.plugins.module_utils.network.asa.argspec.ogs.ogs import (
    OGsArgs,
)
from ansible_collections.cisco.asa.plugins.module_utils.network.asa.rm_templates.ogs import (
    OGsTemplate,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network_template import (
    NetworkTemplate,
)


class OGsFacts(object):
    """ The asa_ogs fact class
    """

    def __init__(self, module, subspec="config", options="options"):

        self._module = module
        self.argument_spec = OGsArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_og_data(self, connection):
        return connection.get("sh running-config object-group")

    def populate_facts(self, connection, ansible_facts, data=None):
        """ Populate the facts for OGs
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_og_data(connection)

        rmmod = NetworkTemplate(lines=data.splitlines(), tmplt=OGsTemplate())
        current = rmmod.parse()

        ogs = []
        object_groups = {
            "icmp-type": "icmp_type",
            "network": "network_object",
            "protocol": "protocol_object",
            "security": "security_group",
            "service": "service_object",
            "user": "user_object",
        }
        if current.get("ogs"):
            for k, v in iteritems(current.get("ogs")):
                obj_gp = {}
                config_dict = {}
                config_dict["object_type"] = k
                config_dict["object_groups"] = []
                for each in iteritems(v):
                    obj_gp["name"] = each[1].pop("name")
                    each[1].pop("object_type")
                    if each[1].get("description"):
                        obj_gp["description"] = each[1].pop("description")
                    if each[1].get("group_object"):
                        obj_gp["group_object"] = each[1].pop("group_object")
                    obj_gp[object_groups.get(k)] = each[1]
                    config_dict["object_groups"].append(obj_gp)
                    obj_gp = {}
                config_dict["object_groups"] = sorted(
                    config_dict["object_groups"],
                    key=lambda k, sk="name": k[sk],
                )
                ogs.append(config_dict)
        # sort the object group list of dict by object_type
        ogs = sorted(ogs, key=lambda i: i["object_type"])
        facts = {}
        params = utils.remove_empties(
            utils.validate_config(self.argument_spec, {"config": ogs})
        )
        facts["ogs"] = params.get("config")

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts



#!/usr/bin/python
# Copyright 2015 WP Engine, Inc. All rights reserved.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: znode
short_description: Create, delete, retrieve, and update znodes using ZooKeeper
description:
    - Create, delete, retrieve, and update znodes using ZooKeeper.
options:
    hosts:
        description:
            - A list of ZooKeeper servers (format '[server]:[port]').
        required: true
    name:
        description:
            - The path of the znode.
        required: true
    value:
        description:
            - The value assigned to the znode.
    op:
        description:
            - An operation to perform. Mutually exclusive with state.
        choices: [ get, wait, list ]
    state:
        description:
            - The state to enforce. Mutually exclusive with op.
        choices: [ present, absent ]
    timeout:
        description:
            - The amount of time to wait for a node to appear.
        default: 300
    recursive:
        description:
            - Recursively delete node and all its children.
        type: bool
        default: 'no'
requirements:
    - kazoo >= 2.1
    - python >= 2.6
author: "Trey Perry (@treyperry)"
'''

EXAMPLES = """
- name: Creating or updating a znode with a given value
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    value: myvalue
    state: present

- name: Getting the value and stat structure for a znode
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    op: get

- name: Listing a particular znode's children
  community.general.znode:
    hosts: 'localhost:2181'
    name: /zookeeper
    op: list

- name: Waiting 20 seconds for a znode to appear at path /mypath
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    op: wait
    timeout: 20

- name: Deleting a znode at path /mypath
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    state: absent

- name: Creating or updating a znode with a given value on a remote Zookeeper
  community.general.znode:
    hosts: 'my-zookeeper-node:2181'
    name: /mypath
    value: myvalue
    state: present
  delegate_to: 127.0.0.1
"""

import time
import traceback

KAZOO_IMP_ERR = None
try:
    from kazoo.client import KazooClient
    from kazoo.handlers.threading import KazooTimeoutError
    KAZOO_INSTALLED = True
except ImportError:
    KAZOO_IMP_ERR = traceback.format_exc()
    KAZOO_INSTALLED = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils._text import to_bytes


def main():
    module = AnsibleModule(
        argument_spec=dict(
            hosts=dict(required=True, type='str'),
            name=dict(required=True, type='str'),
            value=dict(required=False, default=None, type='str'),
            op=dict(required=False, default=None, choices=['get', 'wait', 'list']),
            state=dict(choices=['present', 'absent']),
            timeout=dict(required=False, default=300, type='int'),
            recursive=dict(required=False, default=False, type='bool')
        ),
        supports_check_mode=False
    )

    if not KAZOO_INSTALLED:
        module.fail_json(msg=missing_required_lib('kazoo >= 2.1'), exception=KAZOO_IMP_ERR)

    check = check_params(module.params)
    if not check['success']:
        module.fail_json(msg=check['msg'])

    zoo = KazooCommandProxy(module)
    try:
        zoo.start()
    except KazooTimeoutError:
        module.fail_json(msg='The connection to the ZooKeeper ensemble timed out.')

    command_dict = {
        'op': {
            'get': zoo.get,
            'list': zoo.list,
            'wait': zoo.wait
        },
        'state': {
            'present': zoo.present,
            'absent': zoo.absent
        }
    }

    command_type = 'op' if 'op' in module.params and module.params['op'] is not None else 'state'
    method = module.params[command_type]
    result, result_dict = command_dict[command_type][method]()
    zoo.shutdown()

    if result:
        module.exit_json(**result_dict)
    else:
        module.fail_json(**result_dict)


def check_params(params):
    if not params['state'] and not params['op']:
        return {'success': False, 'msg': 'Please define an operation (op) or a state.'}

    if params['state'] and params['op']:
        return {'success': False, 'msg': 'Please choose an operation (op) or a state, but not both.'}

    return {'success': True}


class KazooCommandProxy():
    def __init__(self, module):
        self.module = module
        self.zk = KazooClient(module.params['hosts'])

    def absent(self):
        return self._absent(self.module.params['name'])

    def exists(self, znode):
        return self.zk.exists(znode)

    def list(self):
        children = self.zk.get_children(self.module.params['name'])
        return True, {'count': len(children), 'items': children, 'msg': 'Retrieved znodes in path.',
                      'znode': self.module.params['name']}

    def present(self):
        return self._present(self.module.params['name'], self.module.params['value'])

    def get(self):
        return self._get(self.module.params['name'])

    def shutdown(self):
        self.zk.stop()
        self.zk.close()

    def start(self):
        self.zk.start()

    def wait(self):
        return self._wait(self.module.params['name'], self.module.params['timeout'])

    def _absent(self, znode):
        if self.exists(znode):
            self.zk.delete(znode, recursive=self.module.params['recursive'])
            return True, {'changed': True, 'msg': 'The znode was deleted.'}
        else:
            return True, {'changed': False, 'msg': 'The znode does not exist.'}

    def _get(self, path):
        if self.exists(path):
            value, zstat = self.zk.get(path)
            stat_dict = {}
            for i in dir(zstat):
                if not i.startswith('_'):
                    attr = getattr(zstat, i)
                    if isinstance(attr, (int, str)):
                        stat_dict[i] = attr
            result = True, {'msg': 'The node was retrieved.', 'znode': path, 'value': value,
                            'stat': stat_dict}
        else:
            result = False, {'msg': 'The requested node does not exist.'}

        return result

    def _present(self, path, value):
        if self.exists(path):
            (current_value, zstat) = self.zk.get(path)
            if value != current_value:
                self.zk.set(path, to_bytes(value))
                return True, {'changed': True, 'msg': 'Updated the znode value.', 'znode': path,
                              'value': value}
            else:
                return True, {'changed': False, 'msg': 'No changes were necessary.', 'znode': path, 'value': value}
        else:
            self.zk.create(path, to_bytes(value), makepath=True)
            return True, {'changed': True, 'msg': 'Created a new znode.', 'znode': path, 'value': value}

    def _wait(self, path, timeout, interval=5):
        lim = time.time() + timeout

        while time.time() < lim:
            if self.exists(path):
                return True, {'msg': 'The node appeared before the configured timeout.',
                              'znode': path, 'timeout': timeout}
            else:
                time.sleep(interval)

        return False, {'msg': 'The node did not appear before the operation timed out.', 'timeout': timeout,
                       'znode': path}


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: net_linkagg
author: Ricardo Carrillo Cruz (@rcarrillocruz)
short_description: (deprecated, removed after 2022-06-01) Manage link aggregation
  groups on network devices
description:
- This module provides declarative management of link aggregation groups on network
  devices.
version_added: 1.0.0
deprecated:
  alternative: Use platform-specific "[netos]_lag_interfaces" module
  why: Updated modules released with more functionality
  removed_at_date: '2022-06-01'
extends_documentation_fragment:
- ansible.netcommon.network_agnostic
options:
  name:
    description:
    - Name of the link aggregation group.
    required: true
  mode:
    description:
    - Mode of the link aggregation group. A value of C(on) will enable LACP. C(active)
      configures the link to actively information about the state of the link, or
      it can be configured in C(passive) mode ie. send link state information only
      when received them from another link.
    default: true
    choices:
    - on
    - active
    - passive
  members:
    description:
    - List of members interfaces of the link aggregation group. The value can be single
      interface or list of interfaces.
    required: true
  min_links:
    description:
    - Minimum members that should be up before bringing up the link aggregation group.
  aggregate:
    description: List of link aggregation definitions.
  purge:
    description:
    - Purge link aggregation groups not defined in the I(aggregate) parameter.
    default: false
  state:
    description:
    - State of the link aggregation group.
    default: present
    choices:
    - present
    - absent
    - up
    - down

"""

EXAMPLES = """
- name: configure link aggregation group
  ansible.netcommon.net_linkagg:
    name: bond0
    members:
    - eth0
    - eth1

- name: remove configuration
  ansible.netcommon.net_linkagg:
    name: bond0
    state: absent

- name: Create aggregate of linkagg definitions
  ansible.netcommon.net_linkagg:
    aggregate:
    - {name: bond0, members: [eth1]}
    - {name: bond1, members: [eth2]}

- name: Remove aggregate of linkagg definitions
  ansible.netcommon.net_linkagg:
    aggregate:
    - name: bond0
    - name: bond1
    state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always, except for the platforms that use Netconf transport to manage the device.
  type: list
  sample:
    - set interfaces bonding bond0
    - set interfaces ethernet eth0 bond-group 'bond0'
    - set interfaces ethernet eth1 bond-group 'bond0'
"""



# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import unittest

from ansible_collections.community.general.plugins.modules.cloud.google.gcp_url_map import _build_path_matchers, _build_url_map_dict


class TestGCPUrlMap(unittest.TestCase):
    """Unit tests for gcp_url_map module."""
    params_dict = {
        'url_map_name': 'foo_url_map_name',
        'description': 'foo_url_map description',
        'host_rules': [
            {
                'description': 'host rules description',
                'hosts': [
                        'www.example.com',
                        'www2.example.com'
                ],
                'path_matcher': 'host_rules_path_matcher'
            }
        ],
        'path_matchers': [
            {
                'name': 'path_matcher_one',
                'description': 'path matcher one',
                'defaultService': 'bes-pathmatcher-one-default',
                'pathRules': [
                        {
                            'service': 'my-one-bes',
                            'paths': [
                                '/',
                                '/aboutus'
                            ]
                        }
                ]
            },
            {
                'name': 'path_matcher_two',
                'description': 'path matcher two',
                'defaultService': 'bes-pathmatcher-two-default',
                'pathRules': [
                        {
                            'service': 'my-two-bes',
                            'paths': [
                                '/webapp',
                                '/graphs'
                            ]
                        }
                ]
            }
        ]
    }

    def test__build_path_matchers(self):
        input_list = [
            {
                'defaultService': 'bes-pathmatcher-one-default',
                'description': 'path matcher one',
                'name': 'path_matcher_one',
                'pathRules': [
                    {
                        'paths': [
                            '/',
                            '/aboutus'
                        ],
                        'service': 'my-one-bes'
                    }
                ]
            },
            {
                'defaultService': 'bes-pathmatcher-two-default',
                'description': 'path matcher two',
                'name': 'path_matcher_two',
                'pathRules': [
                    {
                        'paths': [
                            '/webapp',
                            '/graphs'
                        ],
                        'service': 'my-two-bes'
                    }
                ]
            }
        ]
        expected = [
            {
                'defaultService': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/bes-pathmatcher-one-default',
                'description': 'path matcher one',
                'name': 'path_matcher_one',
                'pathRules': [
                    {
                        'paths': [
                            '/',
                            '/aboutus'
                        ],
                        'service': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/my-one-bes'
                    }
                ]
            },
            {
                'defaultService': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/bes-pathmatcher-two-default',
                'description': 'path matcher two',
                'name': 'path_matcher_two',
                'pathRules': [
                    {
                        'paths': [
                            '/webapp',
                            '/graphs'
                        ],
                        'service': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/my-two-bes'
                    }
                ]
            }
        ]
        actual = _build_path_matchers(input_list, 'my-project')
        self.assertEqual(expected, actual)

    def test__build_url_map_dict(self):

        expected = {
            'description': 'foo_url_map description',
            'hostRules': [
                {
                    'description': 'host rules description',
                    'hosts': [
                        'www.example.com',
                        'www2.example.com'
                    ],
                    'pathMatcher': 'host_rules_path_matcher'
                }
            ],
            'name': 'foo_url_map_name',
            'pathMatchers': [
                {
                    'defaultService': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/bes-pathmatcher-one-default',
                    'description': 'path matcher one',
                    'name': 'path_matcher_one',
                    'pathRules': [
                        {
                            'paths': [
                                '/',
                                '/aboutus'
                            ],
                            'service': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/my-one-bes'
                        }
                    ]
                },
                {
                    'defaultService': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/bes-pathmatcher-two-default',
                    'description': 'path matcher two',
                    'name': 'path_matcher_two',
                    'pathRules': [
                        {
                            'paths': [
                                '/webapp',
                                '/graphs'
                            ],
                            'service': 'https://www.googleapis.com/compute/v1/projects/my-project/global/backendServices/my-two-bes'
                        }
                    ]
                }
            ]
        }
        actual = _build_url_map_dict(self.params_dict, 'my-project')
        self.assertEqual(expected, actual)



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: rax_mon_notification
short_description: Create or delete a Rackspace Cloud Monitoring notification.
description:
- Create or delete a Rackspace Cloud Monitoring notification that specifies a
  channel that can be used to communicate alarms, such as email, webhooks, or
  PagerDuty. Rackspace monitoring module flow | rax_mon_entity -> rax_mon_check ->
  *rax_mon_notification* -> rax_mon_notification_plan -> rax_mon_alarm
options:
  state:
    type: str
    description:
    - Ensure that the notification with this C(label) exists or does not exist.
    choices: ['present', 'absent']
    default: present
  label:
    type: str
    description:
    - Defines a friendly name for this notification. String between 1 and 255
      characters long.
    required: true
  notification_type:
    type: str
    description:
    - A supported notification type.
    choices: ["webhook", "email", "pagerduty"]
    required: true
  details:
    type: dict
    description:
    - Dictionary of key-value pairs used to initialize the notification.
      Required keys and meanings vary with notification type. See
      http://docs.rackspace.com/cm/api/v1.0/cm-devguide/content/
      service-notification-types-crud.html for details.
    required: true
author: Ash Wilson (@smashwilson)
extends_documentation_fragment:
- community.general.rackspace.openstack

'''

EXAMPLES = '''
- name: Monitoring notification example
  gather_facts: False
  hosts: local
  connection: local
  tasks:
  - name: Email me when something goes wrong.
    rax_mon_entity:
      credentials: ~/.rax_pub
      label: omg
      type: email
      details:
        address: me@mailhost.com
    register: the_notification
'''

try:
    import pyrax
    HAS_PYRAX = True
except ImportError:
    HAS_PYRAX = False

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.rax import rax_argument_spec, rax_required_together, setup_rax_module


def notification(module, state, label, notification_type, details):

    if len(label) < 1 or len(label) > 255:
        module.fail_json(msg='label must be between 1 and 255 characters long')

    changed = False
    notification = None

    cm = pyrax.cloud_monitoring
    if not cm:
        module.fail_json(msg='Failed to instantiate client. This typically '
                             'indicates an invalid region or an incorrectly '
                             'capitalized region name.')

    existing = []
    for n in cm.list_notifications():
        if n.label == label:
            existing.append(n)

    if existing:
        notification = existing[0]

    if state == 'present':
        should_update = False
        should_delete = False
        should_create = False

        if len(existing) > 1:
            module.fail_json(msg='%s existing notifications are labelled %s.' %
                                 (len(existing), label))

        if notification:
            should_delete = (notification_type != notification.type)

            should_update = (details != notification.details)

            if should_update and not should_delete:
                notification.update(details=notification.details)
                changed = True

            if should_delete:
                notification.delete()
        else:
            should_create = True

        if should_create:
            notification = cm.create_notification(notification_type,
                                                  label=label, details=details)
            changed = True
    else:
        for n in existing:
            n.delete()
            changed = True

    if notification:
        notification_dict = {
            "id": notification.id,
            "type": notification.type,
            "label": notification.label,
            "details": notification.details
        }
        module.exit_json(changed=changed, notification=notification_dict)
    else:
        module.exit_json(changed=changed)


def main():
    argument_spec = rax_argument_spec()
    argument_spec.update(
        dict(
            state=dict(default='present', choices=['present', 'absent']),
            label=dict(required=True),
            notification_type=dict(required=True, choices=['webhook', 'email', 'pagerduty']),
            details=dict(required=True, type='dict')
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_together=rax_required_together()
    )

    if not HAS_PYRAX:
        module.fail_json(msg='pyrax is required for this module')

    state = module.params.get('state')

    label = module.params.get('label')
    notification_type = module.params.get('notification_type')
    details = module.params.get('details')

    setup_rax_module(module, pyrax)

    notification(module, state, label, notification_type, details)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Fred-Sun, (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_registrationassignment
version_added: '1.3.0'
short_description: Manage Azure RegistrationAssignment instance
description:
    - Create and delete instance of Azure RegistrationAssignment.
options:
    scope:
        description:
            - Scope of the registration assignment. Can be in subscription or group level.
        required: true
        type: str
    registration_assignment_id:
        description:
            - ID of the registration assignment.
            - If is not specified, an UUID will be generated for it.
        type: str
    properties:
        description:
            - Properties of a registration assignment.
        type: dict
        suboptions:
            registration_definition_id:
                description:
                  - Fully qualified path of the registration definition.
                required: true
                type: str
    state:
        description:
            - Assert the state of the RegistrationAssignment.
            - Use C(present) to create or update an RegistrationAssignment and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Fred-Sun (@Fred-Sun)

'''

EXAMPLES = '''
    - name: Delete Registration Assignment
      azure_rm_registrationassignment:
        scope: subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        registration_assignment_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        state: absent


    - name: Create Registration Assignment in subscription level
      azure_rm_registrationassignment:
        scope: subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        registration_assignment_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        properties:
          registration_definition_id: /subscriptions/xxx-xxx/providers/Microsoft.ManagedServices/registrationDefinitions/xxx-xxx


    - name: Create Registration Assignment in resourcegroup level with randomly generating registration_assignment_id
      azure_rm_registrationassignment:
        scope: subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup
        properties:
          registration_definition_id: /subscriptions/xxx-xxx/providers/Microsoft.ManagedServices/registrationDefinitions/xxx-xxx

'''

RETURN = '''
state:
    description:
        - The state info of the registration assignment.
    type: complex
    returned: always
    contains:
        properties:
            description:
                - Properties of a registration assignment.
            returned: always
            type: complex
            contains:
                registration_definition_id:
                    description:
                        - Fully qualified path of the registration definition.
                    returned: always
                    type: str
                    sample: null
        id:
            description:
                - The fully qualified path of the registration assignment.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/providers/Microsoft.ManagedServices/registrationAssignments/xxx-xxx
        type:
            description:
                - Type of the resource.
            returned: always
            type: str
            sample: Microsoft.ManagedServices/registrationAssignments
        name:
            description:
                - Name of the registration assignment.
            returned: always
            type: str
            sample: 9b2895ec-fb1e-4a1e-a978-abd9933d6b20

'''
import uuid
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
try:
    from msrestazure.azure_exceptions import CloudError
    from azure.mgmt.managedservices import ManagedServicesClient
    from msrestazure.azure_operation import AzureOperationPoller
    from msrest.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMRegistrationAssignment(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            scope=dict(
                type='str',
                required=True
            ),
            registration_assignment_id=dict(
                type='str',
            ),
            properties=dict(
                type='dict',
                disposition='/properties',
                options=dict(
                    registration_definition_id=dict(
                        type='str',
                        disposition='registration_definition_id',
                        required=True
                    )
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.scope = None
        self.registration_assignment_id = None
        self.expand_registration_definition = False
        self.body = {}

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMRegistrationAssignment, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                            supports_check_mode=True,
                                                            supports_tags=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)

        old_response = None
        response = None
        if self.registration_assignment_id is None:
            self.registration_assignment_id = str(uuid.uuid4())

        self.mgmt_client = self.get_mgmt_svc_client(ManagedServicesClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager,
                                                    api_version='2019-09-01',
                                                    suppress_subscription_id=True)

        old_response = self.get_resource()

        if not old_response:
            if self.state == 'present':
                self.to_do = Actions.Create
        else:
            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                modifiers = {}
                self.create_compare_modifiers(self.module_arg_spec, '', modifiers)
                self.results['modifiers'] = modifiers
                self.results['compare'] = []
                if not self.default_compare(modifiers, self.body, old_response, '', self.results):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.create_update_resource()
            self.results['state'] = response
        elif self.to_do == Actions.Delete:
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            self.delete_resource()
        else:
            self.results['changed'] = False
            response = old_response
            self.results['state'] = response

        if self.state is 'present':
            if self.results['state'].get('properties', None) is not None:
                registration_definition_id = self.results['state']['properties']['registration_definition_id']
                self.results['state']['properties'].clear()
                self.results['state']['properties']['registration_definition_id'] = registration_definition_id

        return self.results

    def create_update_resource(self):
        try:
            response = self.mgmt_client.registration_assignments.create_or_update(scope=self.scope,
                                                                                  registration_assignment_id=self.registration_assignment_id,
                                                                                  properties=self.body.get('properties', None),
                                                                                  request_body=self.body)
            if isinstance(response, AzureOperationPoller) or isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except CloudError as exc:
            self.log('Error attempting to create the RegistrationAssignment instance.')
            self.fail('Error creating the RegistrationAssignment instance: {0}'.format(str(exc)))
        return response.as_dict()

    def delete_resource(self):
        try:
            response = self.mgmt_client.registration_assignments.delete(scope=self.scope,
                                                                        registration_assignment_id=self.registration_assignment_id)
        except CloudError as e:
            self.log('Error attempting to delete the RegistrationAssignment instance.')
            self.fail('Error deleting the RegistrationAssignment instance: {0}'.format(str(e)))

        return True

    def get_resource(self):
        try:
            response = self.mgmt_client.registration_assignments.get(scope=self.scope,
                                                                     registration_assignment_id=self.registration_assignment_id,
                                                                     expand_registration_definition=self.expand_registration_definition)
        except Exception as e:
            return False
        return response.as_dict()


def main():
    AzureRMRegistrationAssignment()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright: (c) 2018, Red Hat, Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
author:
    - Bryan Gurney (@bgurney-rh)

module: vdo

short_description: Module to control VDO


description:
    - This module controls the VDO dedupe and compression device.
    - VDO, or Virtual Data Optimizer, is a device-mapper target that
      provides inline block-level deduplication, compression, and
      thin provisioning capabilities to primary storage.

options:
    name:
        description:
            - The name of the VDO volume.
        type: str
        required: true
    state:
        description:
            - Whether this VDO volume should be "present" or "absent".
              If a "present" VDO volume does not exist, it will be
              created.  If a "present" VDO volume already exists, it
              will be modified, by updating the configuration, which
              will take effect when the VDO volume is restarted.
              Not all parameters of an existing VDO volume can be
              modified; the "statusparamkeys" list contains the
              parameters that can be modified after creation. If an
              "absent" VDO volume does not exist, it will not be
              removed.
        type: str
        choices: [ absent, present ]
        default: present
    activated:
        description:
            - The "activate" status for a VDO volume.  If this is set
              to "no", the VDO volume cannot be started, and it will
              not start on system startup.  However, on initial
              creation, a VDO volume with "activated" set to "off"
              will be running, until stopped.  This is the default
              behavior of the "vdo create" command; it provides the
              user an opportunity to write a base amount of metadata
              (filesystem, LVM headers, etc.) to the VDO volume prior
              to stopping the volume, and leaving it deactivated
              until ready to use.
        type: bool
    running:
        description:
            - Whether this VDO volume is running.
            - A VDO volume must be activated in order to be started.
        type: bool
    device:
        description:
            - The full path of the device to use for VDO storage.
            - This is required if "state" is "present".
        type: str
    logicalsize:
        description:
            - The logical size of the VDO volume (in megabytes, or
              LVM suffix format).  If not specified for a new volume,
              this defaults to the same size as the underlying storage
              device, which is specified in the 'device' parameter.
              Existing volumes will maintain their size if the
              logicalsize parameter is not specified, or is smaller
              than or identical to the current size.  If the specified
              size is larger than the current size, a growlogical
              operation will be performed.
        type: str
    deduplication:
        description:
            - Configures whether deduplication is enabled.  The
              default for a created volume is 'enabled'.  Existing
              volumes will maintain their previously configured
              setting unless a different value is specified in the
              playbook.
        type: str
        choices: [ disabled, enabled ]
    compression:
        description:
            - Configures whether compression is enabled.  The default
              for a created volume is 'enabled'.  Existing volumes
              will maintain their previously configured setting unless
              a different value is specified in the playbook.
        type: str
        choices: [ disabled, enabled ]
    blockmapcachesize:
        description:
            - The amount of memory allocated for caching block map
              pages, in megabytes (or may be issued with an LVM-style
              suffix of K, M, G, or T).  The default (and minimum)
              value is 128M.  The value specifies the size of the
              cache; there is a 15% memory usage overhead. Each 1.25G
              of block map covers 1T of logical blocks, therefore a
              small amount of block map cache memory can cache a
              significantly large amount of block map data.  Existing
              volumes will maintain their previously configured
              setting unless a different value is specified in the
              playbook.
        type: str
    readcache:
        description:
            - Enables or disables the read cache.  The default is
              'disabled'.  Choosing 'enabled' enables a read cache
              which may improve performance for workloads of high
              deduplication, read workloads with a high level of
              compression, or on hard disk storage.  Existing
              volumes will maintain their previously configured
              setting unless a different value is specified in the
              playbook.
            - The read cache feature is available in VDO 6.1 and older.
        type: str
        choices: [ disabled, enabled ]
    readcachesize:
        description:
            - Specifies the extra VDO device read cache size in
              megabytes.  This is in addition to a system-defined
              minimum.  Using a value with a suffix of K, M, G, or T
              is optional.  The default value is 0.  1.125 MB of
              memory per bio thread will be used per 1 MB of read
              cache specified (for example, a VDO volume configured
              with 4 bio threads will have a read cache memory usage
              overhead of 4.5 MB per 1 MB of read cache specified).
              Existing volumes will maintain their previously
              configured setting unless a different value is specified
              in the playbook.
            - The read cache feature is available in VDO 6.1 and older.
        type: str
    emulate512:
        description:
            - Enables 512-byte emulation mode, allowing drivers or
              filesystems to access the VDO volume at 512-byte
              granularity, instead of the default 4096-byte granularity.
              Default is 'disabled'; only recommended when a driver
              or filesystem requires 512-byte sector level access to
              a device.  This option is only available when creating
              a new volume, and cannot be changed for an existing
              volume.
        type: bool
        default: false
    growphysical:
        description:
            - Specifies whether to attempt to execute a growphysical
              operation, if there is enough unused space on the
              device.  A growphysical operation will be executed if
              there is at least 64 GB of free space, relative to the
              previous physical size of the affected VDO volume.
        type: bool
        default: false
    slabsize:
        description:
            - The size of the increment by which the physical size of
              a VDO volume is grown, in megabytes (or may be issued
              with an LVM-style suffix of K, M, G, or T).  Must be a
              power of two between 128M and 32G.  The default is 2G,
              which supports volumes having a physical size up to 16T.
              The maximum, 32G, supports a physical size of up to 256T.
              This option is only available when creating a new
              volume, and cannot be changed for an existing volume.
        type: str
    writepolicy:
        description:
            - Specifies the write policy of the VDO volume.  The
              'sync' mode acknowledges writes only after data is on
              stable storage.  The 'async' mode acknowledges writes
              when data has been cached for writing to stable
              storage.  The default (and highly recommended) 'auto'
              mode checks the storage device to determine whether it
              supports flushes.  Devices that support flushes will
              result in a VDO volume in 'async' mode, while devices
              that do not support flushes will run in sync mode.
              Existing volumes will maintain their previously
              configured setting unless a different value is
              specified in the playbook.
        type: str
        choices: [ async, auto, sync ]
    indexmem:
        description:
            - Specifies the amount of index memory in gigabytes.  The
              default is 0.25.  The special decimal values 0.25, 0.5,
              and 0.75 can be used, as can any positive integer.
              This option is only available when creating a new
              volume, and cannot be changed for an existing volume.
        type: str
    indexmode:
        description:
            - Specifies the index mode of the Albireo index.  The
              default is 'dense', which has a deduplication window of
              1 GB of index memory per 1 TB of incoming data,
              requiring 10 GB of index data on persistent storage.
              The 'sparse' mode has a deduplication window of 1 GB of
              index memory per 10 TB of incoming data, but requires
              100 GB of index data on persistent storage.  This option
              is only available when creating a new volume, and cannot
              be changed for an existing volume.
        type: str
        choices: [ dense, sparse ]
    ackthreads:
        description:
            - Specifies the number of threads to use for
              acknowledging completion of requested VDO I/O operations.
              Valid values are integer values from 1 to 100 (lower
              numbers are preferable due to overhead).  The default is
              1.  Existing volumes will maintain their previously
              configured setting unless a different value is specified
              in the playbook.
        type: str
    biothreads:
        description:
            - Specifies the number of threads to use for submitting I/O
              operations to the storage device.  Valid values are
              integer values from 1 to 100 (lower numbers are
              preferable due to overhead).  The default is 4.
              Existing volumes will maintain their previously
              configured setting unless a different value is specified
              in the playbook.
        type: str
    cputhreads:
        description:
            - Specifies the number of threads to use for CPU-intensive
              work such as hashing or compression.  Valid values are
              integer values from 1 to 100 (lower numbers are
              preferable due to overhead).  The default is 2.
              Existing volumes will maintain their previously
              configured setting unless a different value is specified
              in the playbook.
        type: str
    logicalthreads:
        description:
            - Specifies the number of threads across which to
              subdivide parts of the VDO processing based on logical
              block addresses.  Valid values are integer values from
              1 to 100 (lower numbers are preferable due to overhead).
              The default is 1.  Existing volumes will maintain their
              previously configured setting unless a different value
              is specified in the playbook.
        type: str
    physicalthreads:
        description:
            - Specifies the number of threads across which to
              subdivide parts of the VDO processing based on physical
              block addresses.  Valid values are integer values from
              1 to 16 (lower numbers are preferable due to overhead).
              The physical space used by the VDO volume must be
              larger than (slabsize * physicalthreads).  The default
              is 1.  Existing volumes will maintain their previously
              configured setting unless a different value is specified
              in the playbook.
        type: str
notes:
  - In general, the default thread configuration should be used.
requirements:
  - PyYAML
  - kmod-kvdo
  - vdo
'''

EXAMPLES = r'''
- name: Create 2 TB VDO volume vdo1 on device /dev/md0
  community.general.vdo:
    name: vdo1
    state: present
    device: /dev/md0
    logicalsize: 2T

- name: Remove VDO volume vdo1
  community.general.vdo:
    name: vdo1
    state: absent
'''

RETURN = r'''#  '''

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
import re
import traceback

YAML_IMP_ERR = None
try:
    import yaml
    HAS_YAML = True
except ImportError:
    YAML_IMP_ERR = traceback.format_exc()
    HAS_YAML = False


# Generate a list of VDO volumes, whether they are running or stopped.
#
# @param module  The AnsibleModule object.
# @param vdocmd  The path of the 'vdo' command.
#
# @return vdolist  A list of currently created VDO volumes.
def inventory_vdos(module, vdocmd):
    rc, vdostatusout, err = module.run_command("%s status" % (vdocmd))

    # if rc != 0:
    #   module.fail_json(msg="Inventorying VDOs failed: %s"
    #                        % vdostatusout, rc=rc, err=err)

    vdolist = []

    if (rc == 2 and
            re.findall(r"vdoconf.yml does not exist", err, re.MULTILINE)):
        # If there is no /etc/vdoconf.yml file, assume there are no
        # VDO volumes. Return an empty list of VDO volumes.
        return vdolist

    if rc != 0:
        module.fail_json(msg="Inventorying VDOs failed: %s"
                             % vdostatusout, rc=rc, err=err)

    vdostatusyaml = yaml.load(vdostatusout)
    if vdostatusyaml is None:
        return vdolist

    vdoyamls = vdostatusyaml['VDOs']

    if vdoyamls is not None:
        vdolist = vdoyamls.keys()

    return vdolist


def list_running_vdos(module, vdocmd):
    rc, vdolistout, err = module.run_command("%s list" % (vdocmd))
    runningvdolist = filter(None, vdolistout.split('\n'))
    return runningvdolist


# Generate a string containing options to pass to the 'VDO' command.
# Note that a 'create' operation will pass more options than a
# 'modify' operation.
#
# @param params  A dictionary of parameters, and their values
#                (values of 'None' and/or nonexistent values are ignored).
#
# @return vdocmdoptions  A string to be used in a 'vdo <action>' command.
def start_vdo(module, vdoname, vdocmd):
    rc, out, err = module.run_command("%s start --name=%s" % (vdocmd, vdoname))
    if rc == 0:
        module.log("started VDO volume %s" % vdoname)

    return rc


def stop_vdo(module, vdoname, vdocmd):
    rc, out, err = module.run_command("%s stop --name=%s" % (vdocmd, vdoname))
    if rc == 0:
        module.log("stopped VDO volume %s" % vdoname)

    return rc


def activate_vdo(module, vdoname, vdocmd):
    rc, out, err = module.run_command("%s activate --name=%s"
                                      % (vdocmd, vdoname))
    if rc == 0:
        module.log("activated VDO volume %s" % vdoname)

    return rc


def deactivate_vdo(module, vdoname, vdocmd):
    rc, out, err = module.run_command("%s deactivate --name=%s"
                                      % (vdocmd, vdoname))
    if rc == 0:
        module.log("deactivated VDO volume %s" % vdoname)

    return rc


def add_vdooptions(params):
    vdocmdoptions = ""
    options = []

    if ('logicalsize' in params) and (params['logicalsize'] is not None):
        options.append("--vdoLogicalSize=" + params['logicalsize'])

    if (('blockmapcachesize' in params) and
            (params['blockmapcachesize'] is not None)):
        options.append("--blockMapCacheSize=" + params['blockmapcachesize'])

    if ('readcache' in params) and (params['readcache'] == 'enabled'):
        options.append("--readCache=enabled")

    if ('readcachesize' in params) and (params['readcachesize'] is not None):
        options.append("--readCacheSize=" + params['readcachesize'])

    if ('slabsize' in params) and (params['slabsize'] is not None):
        options.append("--vdoSlabSize=" + params['slabsize'])

    if ('emulate512' in params) and (params['emulate512']):
        options.append("--emulate512=enabled")

    if ('indexmem' in params) and (params['indexmem'] is not None):
        options.append("--indexMem=" + params['indexmem'])

    if ('indexmode' in params) and (params['indexmode'] == 'sparse'):
        options.append("--sparseIndex=enabled")

    # Entering an invalid thread config results in a cryptic
    # 'Could not set up device mapper for %s' error from the 'vdo'
    # command execution.  The dmsetup module on the system will
    # output a more helpful message, but one would have to log
    # onto that system to read the error.  For now, heed the thread
    # limit warnings in the DOCUMENTATION section above.
    if ('ackthreads' in params) and (params['ackthreads'] is not None):
        options.append("--vdoAckThreads=" + params['ackthreads'])

    if ('biothreads' in params) and (params['biothreads'] is not None):
        options.append("--vdoBioThreads=" + params['biothreads'])

    if ('cputhreads' in params) and (params['cputhreads'] is not None):
        options.append("--vdoCpuThreads=" + params['cputhreads'])

    if ('logicalthreads' in params) and (params['logicalthreads'] is not None):
        options.append("--vdoLogicalThreads=" + params['logicalthreads'])

    if (('physicalthreads' in params) and
            (params['physicalthreads'] is not None)):
        options.append("--vdoPhysicalThreads=" + params['physicalthreads'])

    vdocmdoptions = ' '.join(options)
    return vdocmdoptions


def run_module():

    # Define the available arguments/parameters that a user can pass to
    # the module.
    # Defaults for VDO parameters are None, in order to facilitate
    # the detection of parameters passed from the playbook.
    # Creation param defaults are determined by the creation section.

    module_args = dict(
        name=dict(type='str', required=True),
        state=dict(type='str', default='present', choices=['absent', 'present']),
        activated=dict(type='bool'),
        running=dict(type='bool'),
        growphysical=dict(type='bool', default=False),
        device=dict(type='str'),
        logicalsize=dict(type='str'),
        deduplication=dict(type='str', choices=['disabled', 'enabled']),
        compression=dict(type='str', choices=['disabled', 'enabled']),
        blockmapcachesize=dict(type='str'),
        readcache=dict(type='str', choices=['disabled', 'enabled']),
        readcachesize=dict(type='str'),
        emulate512=dict(type='bool', default=False),
        slabsize=dict(type='str'),
        writepolicy=dict(type='str', choices=['async', 'auto', 'sync']),
        indexmem=dict(type='str'),
        indexmode=dict(type='str', choices=['dense', 'sparse']),
        ackthreads=dict(type='str'),
        biothreads=dict(type='str'),
        cputhreads=dict(type='str'),
        logicalthreads=dict(type='str'),
        physicalthreads=dict(type='str')
    )

    # Seed the result dictionary in the object.  There will be an
    # 'invocation' dictionary added with 'module_args' (arguments
    # given).
    result = dict(
        changed=False,
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=False,
    )

    if not HAS_YAML:
        module.fail_json(msg=missing_required_lib('PyYAML'), exception=YAML_IMP_ERR)

    vdocmd = module.get_bin_path("vdo", required=True)
    if not vdocmd:
        module.fail_json(msg='VDO is not installed.', **result)

    # Print a pre-run list of VDO volumes in the result object.
    vdolist = inventory_vdos(module, vdocmd)

    runningvdolist = list_running_vdos(module, vdocmd)

    # Collect the name of the desired VDO volume, and its state.  These will
    # determine what to do.
    desiredvdo = module.params['name']
    state = module.params['state']

    # Create a desired VDO volume that doesn't exist yet.
    if (desiredvdo not in vdolist) and (state == 'present'):
        device = module.params['device']
        if device is None:
            module.fail_json(msg="Creating a VDO volume requires specifying "
                                 "a 'device' in the playbook.")

        # Create a dictionary of the options from the AnsibleModule
        # parameters, compile the vdo command options, and run "vdo create"
        # with those options.
        # Since this is a creation of a new VDO volume, it will contain all
        # all of the parameters given by the playbook; the rest will
        # assume default values.
        options = module.params
        vdocmdoptions = add_vdooptions(options)
        rc, out, err = module.run_command("%s create --name=%s --device=%s %s"
                                          % (vdocmd, desiredvdo, device,
                                             vdocmdoptions))
        if rc == 0:
            result['changed'] = True
        else:
            module.fail_json(msg="Creating VDO %s failed."
                             % desiredvdo, rc=rc, err=err)

        if (module.params['compression'] == 'disabled'):
            rc, out, err = module.run_command("%s disableCompression --name=%s"
                                              % (vdocmd, desiredvdo))

        if ((module.params['deduplication'] is not None) and
                module.params['deduplication'] == 'disabled'):
            rc, out, err = module.run_command("%s disableDeduplication "
                                              "--name=%s"
                                              % (vdocmd, desiredvdo))

        if module.params['activated'] == 'no':
            deactivate_vdo(module, desiredvdo, vdocmd)

        if module.params['running'] == 'no':
            stop_vdo(module, desiredvdo, vdocmd)

        # Print a post-run list of VDO volumes in the result object.
        vdolist = inventory_vdos(module, vdocmd)
        module.log("created VDO volume %s" % desiredvdo)
        module.exit_json(**result)

    # Modify the current parameters of a VDO that exists.
    if (desiredvdo in vdolist) and (state == 'present'):
        rc, vdostatusoutput, err = module.run_command("%s status" % (vdocmd))
        vdostatusyaml = yaml.load(vdostatusoutput)

        # An empty dictionary to contain dictionaries of VDO statistics
        processedvdos = {}

        vdoyamls = vdostatusyaml['VDOs']
        if vdoyamls is not None:
            processedvdos = vdoyamls

        # The 'vdo status' keys that are currently modifiable.
        statusparamkeys = ['Acknowledgement threads',
                           'Bio submission threads',
                           'Block map cache size',
                           'CPU-work threads',
                           'Logical threads',
                           'Physical threads',
                           'Read cache',
                           'Read cache size',
                           'Configured write policy',
                           'Compression',
                           'Deduplication']

        # A key translation table from 'vdo status' output to Ansible
        # module parameters.  This covers all of the 'vdo status'
        # parameter keys that could be modified with the 'vdo'
        # command.
        vdokeytrans = {
            'Logical size': 'logicalsize',
            'Compression': 'compression',
            'Deduplication': 'deduplication',
            'Block map cache size': 'blockmapcachesize',
            'Read cache': 'readcache',
            'Read cache size': 'readcachesize',
            'Configured write policy': 'writepolicy',
            'Acknowledgement threads': 'ackthreads',
            'Bio submission threads': 'biothreads',
            'CPU-work threads': 'cputhreads',
            'Logical threads': 'logicalthreads',
            'Physical threads': 'physicalthreads'
        }

        # Build a dictionary of the current VDO status parameters, with
        # the keys used by VDO.  (These keys will be converted later.)
        currentvdoparams = {}

        # Build a "lookup table" dictionary containing a translation table
        # of the parameters that can be modified
        modtrans = {}

        for statfield in statusparamkeys:
            if statfield in processedvdos[desiredvdo]:
                currentvdoparams[statfield] = processedvdos[desiredvdo][statfield]

            modtrans[statfield] = vdokeytrans[statfield]

        # Build a dictionary of current parameters formatted with the
        # same keys as the AnsibleModule parameters.
        currentparams = {}
        for paramkey in modtrans.keys():
            currentparams[modtrans[paramkey]] = modtrans[paramkey]

        diffparams = {}

        # Check for differences between the playbook parameters and the
        # current parameters.  This will need a comparison function;
        # since AnsibleModule params are all strings, compare them as
        # strings (but if it's None; skip).
        for key in currentparams.keys():
            if module.params[key] is not None:
                if str(currentparams[key]) != module.params[key]:
                    diffparams[key] = module.params[key]

        if diffparams:
            vdocmdoptions = add_vdooptions(diffparams)
            if vdocmdoptions:
                rc, out, err = module.run_command("%s modify --name=%s %s"
                                                  % (vdocmd,
                                                     desiredvdo,
                                                     vdocmdoptions))
                if rc == 0:
                    result['changed'] = True
                else:
                    module.fail_json(msg="Modifying VDO %s failed."
                                     % desiredvdo, rc=rc, err=err)

            if 'deduplication' in diffparams.keys():
                dedupemod = diffparams['deduplication']
                if dedupemod == 'disabled':
                    rc, out, err = module.run_command("%s "
                                                      "disableDeduplication "
                                                      "--name=%s"
                                                      % (vdocmd, desiredvdo))

                    if rc == 0:
                        result['changed'] = True
                    else:
                        module.fail_json(msg="Changing deduplication on "
                                             "VDO volume %s failed."
                                         % desiredvdo, rc=rc, err=err)

                if dedupemod == 'enabled':
                    rc, out, err = module.run_command("%s "
                                                      "enableDeduplication "
                                                      "--name=%s"
                                                      % (vdocmd, desiredvdo))

                    if rc == 0:
                        result['changed'] = True
                    else:
                        module.fail_json(msg="Changing deduplication on "
                                             "VDO volume %s failed."
                                         % desiredvdo, rc=rc, err=err)

            if 'compression' in diffparams.keys():
                compressmod = diffparams['compression']
                if compressmod == 'disabled':
                    rc, out, err = module.run_command("%s disableCompression "
                                                      "--name=%s"
                                                      % (vdocmd, desiredvdo))

                    if rc == 0:
                        result['changed'] = True
                    else:
                        module.fail_json(msg="Changing compression on "
                                             "VDO volume %s failed."
                                         % desiredvdo, rc=rc, err=err)

                if compressmod == 'enabled':
                    rc, out, err = module.run_command("%s enableCompression "
                                                      "--name=%s"
                                                      % (vdocmd, desiredvdo))

                    if rc == 0:
                        result['changed'] = True
                    else:
                        module.fail_json(msg="Changing compression on "
                                             "VDO volume %s failed."
                                         % desiredvdo, rc=rc, err=err)

            if 'writepolicy' in diffparams.keys():
                writepolmod = diffparams['writepolicy']
                if writepolmod == 'auto':
                    rc, out, err = module.run_command("%s "
                                                      "changeWritePolicy "
                                                      "--name=%s "
                                                      "--writePolicy=%s"
                                                      % (vdocmd,
                                                         desiredvdo,
                                                         writepolmod))

                    if rc == 0:
                        result['changed'] = True
                    else:
                        module.fail_json(msg="Changing write policy on "
                                             "VDO volume %s failed."
                                         % desiredvdo, rc=rc, err=err)

                if writepolmod == 'sync':
                    rc, out, err = module.run_command("%s "
                                                      "changeWritePolicy "
                                                      "--name=%s "
                                                      "--writePolicy=%s"
                                                      % (vdocmd,
                                                         desiredvdo,
                                                         writepolmod))

                    if rc == 0:
                        result['changed'] = True
                    else:
                        module.fail_json(msg="Changing write policy on "
                                             "VDO volume %s failed."
                                         % desiredvdo, rc=rc, err=err)

                if writepolmod == 'async':
                    rc, out, err = module.run_command("%s "
                                                      "changeWritePolicy "
                                                      "--name=%s "
                                                      "--writePolicy=%s"
                                                      % (vdocmd,
                                                         desiredvdo,
                                                         writepolmod))

                    if rc == 0:
                        result['changed'] = True
                    else:
                        module.fail_json(msg="Changing write policy on "
                                             "VDO volume %s failed."
                                         % desiredvdo, rc=rc, err=err)

        # Process the size parameters, to determine of a growPhysical or
        # growLogical operation needs to occur.
        sizeparamkeys = ['Logical size', ]

        currentsizeparams = {}
        sizetrans = {}
        for statfield in sizeparamkeys:
            currentsizeparams[statfield] = processedvdos[desiredvdo][statfield]
            sizetrans[statfield] = vdokeytrans[statfield]

        sizeparams = {}
        for paramkey in currentsizeparams.keys():
            sizeparams[sizetrans[paramkey]] = currentsizeparams[paramkey]

        diffsizeparams = {}
        for key in sizeparams.keys():
            if module.params[key] is not None:
                if str(sizeparams[key]) != module.params[key]:
                    diffsizeparams[key] = module.params[key]

        if module.params['growphysical']:
            physdevice = module.params['device']
            rc, devsectors, err = module.run_command("blockdev --getsz %s"
                                                     % (physdevice))
            devblocks = (int(devsectors) / 8)
            dmvdoname = ('/dev/mapper/' + desiredvdo)
            currentvdostats = (processedvdos[desiredvdo]
                                            ['VDO statistics']
                                            [dmvdoname])
            currentphysblocks = currentvdostats['physical blocks']

            # Set a growPhysical threshold to grow only when there is
            # guaranteed to be more than 2 slabs worth of unallocated
            # space on the device to use.  For now, set to device
            # size + 64 GB, since 32 GB is the largest possible
            # slab size.
            growthresh = devblocks + 16777216

            if currentphysblocks > growthresh:
                result['changed'] = True
                rc, out, err = module.run_command("%s growPhysical --name=%s"
                                                  % (vdocmd, desiredvdo))

        if 'logicalsize' in diffsizeparams.keys():
            result['changed'] = True
            vdocmdoptions = ("--vdoLogicalSize=" +
                             diffsizeparams['logicalsize'])
            rc, out, err = module.run_command("%s growLogical --name=%s %s"
                                              % (vdocmd,
                                                 desiredvdo,
                                                 vdocmdoptions))

        vdoactivatestatus = processedvdos[desiredvdo]['Activate']

        if ((module.params['activated'] == 'no') and
                (vdoactivatestatus == 'enabled')):
            deactivate_vdo(module, desiredvdo, vdocmd)
            if not result['changed']:
                result['changed'] = True

        if ((module.params['activated'] == 'yes') and
                (vdoactivatestatus == 'disabled')):
            activate_vdo(module, desiredvdo, vdocmd)
            if not result['changed']:
                result['changed'] = True

        if ((module.params['running'] == 'no') and
                (desiredvdo in runningvdolist)):
            stop_vdo(module, desiredvdo, vdocmd)
            if not result['changed']:
                result['changed'] = True

        # Note that a disabled VDO volume cannot be started by the
        # 'vdo start' command, by design.  To accurately track changed
        # status, don't try to start a disabled VDO volume.
        # If the playbook contains 'activated: yes', assume that
        # the activate_vdo() operation succeeded, as 'vdoactivatestatus'
        # will have the activated status prior to the activate_vdo()
        # call.
        if (((vdoactivatestatus == 'enabled') or
                (module.params['activated'] == 'yes')) and
                (module.params['running'] == 'yes') and
                (desiredvdo not in runningvdolist)):
            start_vdo(module, desiredvdo, vdocmd)
            if not result['changed']:
                result['changed'] = True

        # Print a post-run list of VDO volumes in the result object.
        vdolist = inventory_vdos(module, vdocmd)
        if diffparams:
            module.log("modified parameters of VDO volume %s" % desiredvdo)

        module.exit_json(**result)

    # Remove a desired VDO that currently exists.
    if (desiredvdo in vdolist) and (state == 'absent'):
        rc, out, err = module.run_command("%s remove --name=%s"
                                          % (vdocmd, desiredvdo))
        if rc == 0:
            result['changed'] = True
        else:
            module.fail_json(msg="Removing VDO %s failed."
                             % desiredvdo, rc=rc, err=err)

        # Print a post-run list of VDO volumes in the result object.
        vdolist = inventory_vdos(module, vdocmd)
        module.log("removed VDO volume %s" % desiredvdo)
        module.exit_json(**result)

    # fall through
    # The state for the desired VDO volume was absent, and it does
    # not exist. Print a post-run list of VDO volumes in the result
    # object.
    vdolist = inventory_vdos(module, vdocmd)
    module.log("received request to remove non-existent VDO volume %s"
               % desiredvdo)

    module.exit_json(**result)


def main():
    run_module()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

"""
The Ospf_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network_template import (
    NetworkTemplate,
)


def _tmplt_ospf_interface(config_data):
    command = "interface {name}".format(**config_data)
    return command


def _tmplt_ospf_interface_process(config_data):
    if "process" in config_data:
        if config_data.get("afi") == "ipv4":
            command = "ip ospf {id} area {area_id}".format(
                **config_data["process"]
            )
        elif config_data.get("afi") == "ipv6":
            command = "ipv6 ospf {id} area {area_id}".format(
                **config_data["process"]
            )
        if "secondaries" in config_data["process"]:
            command += " secondaries"
    return command


def _tmplt_ip_ospf_authentication(config_data):
    if "authentication" in config_data:
        if config_data.get("afi") == "ipv4":
            command = "ip ospf authentication"
        elif config_data.get("afi") == "ipv6":
            command = "ipv6 ospf authentication"
        if "key_chain" in config_data["authentication"]:
            command += " key-chain {key_chain}".format(
                **config_data["authentication"]
            )
        elif "message_digest" in config_data["authentication"]:
            command += " message-digest"
        elif "null" in config_data["authentication"]:
            command += " null"
    return command


def _tmplt_ip_ospf_cost(config_data):
    if "cost" in config_data:
        if config_data.get("afi") == "ipv4":
            command = "ip ospf cost {interface_cost}".format(
                **config_data["cost"]
            )
        elif config_data.get("afi") == "ipv6":
            command = "ipv6 ospf cost"
            if "interface_cost" in config_data["cost"]:
                command = "ipv6 ospf cost {interface_cost}".format(
                    **config_data["cost"]
                )
            if "dynamic_cost" in config_data["cost"]:
                if "default" in config_data["cost"]["dynamic_cost"]:
                    command += " dynamic default {default}".format(
                        **config_data["cost"]["dynamic_cost"]
                    )
                elif "hysteresis" in config_data["cost"]["dynamic_cost"]:
                    command += " dynamic hysteresis"
                    if (
                        "percent"
                        in config_data["cost"]["dynamic_cost"]["hysteresis"]
                    ):
                        command += " percent {percent}".format(
                            **config_data["cost"]["dynamic_cost"]["hysteresis"]
                        )
                    elif (
                        "threshold"
                        in config_data["cost"]["dynamic_cost"]["hysteresis"]
                    ):
                        command += " threshold {threshold}".format(
                            **config_data["cost"]["dynamic_cost"]["hysteresis"]
                        )
                elif "weight" in config_data["cost"]["dynamic_cost"]:
                    command += " dynamic weight"
                    if (
                        "l2_factor"
                        in config_data["cost"]["dynamic_cost"]["weight"]
                    ):
                        command += " L2-factor {l2_factor}".format(
                            **config_data["cost"]["dynamic_cost"]["weight"]
                        )
                    elif (
                        "latency"
                        in config_data["cost"]["dynamic_cost"]["weight"]
                    ):
                        command += " latency {latency}".format(
                            **config_data["cost"]["dynamic_cost"]["weight"]
                        )
                    elif (
                        "oc" in config_data["cost"]["dynamic_cost"]["weight"]
                        and config_data["cost"]["dynamic_cost"]["weight"]["oc"]
                    ):
                        command += " oc cdr"
                    elif (
                        "resources"
                        in config_data["cost"]["dynamic_cost"]["weight"]
                    ):
                        command += " resources {resources}".format(
                            **config_data["cost"]["dynamic_cost"]["weight"]
                        )
                    elif (
                        "throughput"
                        in config_data["cost"]["dynamic_cost"]["weight"]
                    ):
                        command += " throughput {throughput}".format(
                            **config_data["cost"]["dynamic_cost"]["weight"]
                        )
    return command


def _tmplt_ip_ospf_dead_interval(config_data):
    if "dead_interval" in config_data:
        if config_data.get("afi") == "ipv4":
            command = "ip ospf dead-interval"
            if "time" in config_data["dead_interval"]:
                command += " {time}".format(**config_data["dead_interval"])
            elif "minimal" in config_data["dead_interval"]:
                command += " minimal hello-multiplier {minimal}".format(
                    **config_data["dead_interval"]
                )
        elif config_data.get("afi") == "ipv6":
            command = "ipv6 ospf dead-interval {time}".format(
                **config_data["dead_interval"]
            )
    return command


def _tmplt_ip_ospf_demand_circuit(config_data):
    if "demand_circuit" in config_data:
        if config_data.get("afi") == "ipv4":
            command = "ip ospf demand-circuit"
            if config_data["demand_circuit"]["ignore"]:
                command += " ignore"
            elif config_data["demand_circuit"]["enable"]:
                return command
        elif config_data.get("afi") == "ipv6":
            command = "ipv6 ospf demand-circuit"
            if config_data["demand_circuit"]["enable"]:
                return command
            elif config_data["demand_circuit"]["ignore"]:
                command += " ignore"
            elif config_data["demand_circuit"]["disable"]:
                command += " disable"
    return command


def _tmplt_ip_ospf_manet(config_data):
    if "manet" in config_data:
        command = "ipv6 ospf manet peering"
        if "cost" in config_data["manet"]:
            command += " cost"
            if "percent" in config_data["manet"]["cost"]:
                command += " percent {percent}".format(
                    **config_data["manet"]["cost"]
                )
            elif "threshold" in config_data["manet"]["cost"]:
                command += " threshold {threshold}".format(
                    **config_data["manet"]["cost"]
                )
        elif "link_metrics" in config_data["manet"]:
            command += " link-metrics"
            if "cost_threshold" in config_data["manet"]["link_metrics"]:
                command += " {cost_threshold}".format(
                    **config_data["manet"]["link_metrics"]
                )
    return command


def _tmplt_ip_ospf_multi_area(config_data):
    if "multi_area" in config_data:
        command = "ip ospf multi-area {id}".format(**config_data["multi_area"])
        if "cost" in config_data["multi_area"]:
            command += " cost {cost}".format(**config_data["multi_area"])
    return command


def _tmplt_ip_ospf_neighbor(config_data):
    if "neighbor" in config_data:
        command = "ipv6 ospf neighbor {address}".format(
            **config_data["neighbor"]
        )
    if "cost" in config_data["neighbor"]:
        command += " cost {cost}".format(**config_data["neighbor"])
    if (
        "database_filter" in config_data["neighbor"]
        and config_data["neighbor"]["database_filter"]
    ):
        command += " database-filter all out"
    if "poll_interval" in config_data["neighbor"]:
        command += " poll-interval {poll_interval}".format(
            **config_data["neighbor"]["poll_interval"]
        )
    if "priority" in config_data["neighbor"]:
        command += " priority {priority}".format(
            **config_data["neighbor"]["priority"]
        )
    return command


def _tmplt_ip_ospf_network(config_data):
    if "network" in config_data:
        if config_data.get("afi") == "ipv4":
            command = "ip ospf network"
        elif config_data.get("afi") == "ipv6":
            command = "ipv6 ospf network"
        if "broadcast" in config_data["network"]:
            command += " broadcast"
        if "manet" in config_data["network"]:
            command += " manet"
        if "non_broadcast" in config_data["network"]:
            command += " non-broadcast"
        if "point_to_multipoint" in config_data["network"]:
            command += " point-to-multipoint"
        if "point_to_point" in config_data["network"]:
            command += " point-to-point"
    return command


def _tmplt_ip_ospf_ttl_security(config_data):
    if "ttl_security" in config_data:
        command = "ip ospf ttl-security"
        if "hops" in config_data["ttl_security"]:
            command += " hops {hops}".format(**config_data["ttl_security"])
    return command


class Ospf_InterfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None):
        super(Ospf_InterfacesTemplate, self).__init__(lines=lines, tmplt=self)

    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""
                  ^interface
                  \s(?P<name>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "interface {{ name }}",
            "result": {
                "{{ name }}": {"name": "{{ name }}", "address_family": {}}
            },
            "shared": True,
        },
        {
            "name": "process",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<id>\d+)*
                \s*(?P<area>area\s\d+)*
                \s*(?P<secondaries>secondaries)*

                \s*(?P<instance>instance*\s*\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_interface_process,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "process": {
                                "id": "{{ id }}",
                                "area_id": "{{ area.split(' ')[1] }}",
                                "secondaries": "{{ True if secondaries is defined }}",
                                "instance_id": "{{ instance.split(' ')[1]}}",
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "adjacency",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<adjacency>adjacency*\s*stagger)*
                \s*(?P<disable>disable)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} adjacency stagger disable",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "adjacency": "{{ True if adjacency is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "authentication",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*authentication*
                \s*(?P<key_chain>key-chain*\s*\S+)*
                \s*(?P<message_digest>message-digest)*
                \s*(?P<null>null)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_authentication,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "authentication": {
                                "key_chain": "{{ key_chain.split(' ')[1] }}",
                                "message_digest": "{{ True if message_digest is defined }}",
                                "null": "{{ True if null is defined }}",
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<bfd>bfd)*
                \s*(?P<disable>disable)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} bfd",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "bfd": "{{ True if bfd is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "cost_ip",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)*
                \s*ospf*
                \s*(?P<cost>cost*\s*\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_cost,
            "compval": "cost.interface_cost",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "cost": {
                                "interface_cost": "{{ cost.split(' ')[1] }}"
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "cost_ipv6_dynamic_cost",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ipv6)*
                \s*ospf*
                \s*cost*
                \s*(?P<interface_cost>\d+)*
                \s*dynamic*
                \s*(?P<default>default*\s*\d+)*
                \s*(?P<hysteresis>hysteresis*\s*)*
                \s*(?P<h_params>(percent|threshold)*\s*\d+)*
                \s*(?P<weight>weight)*
                \s*(?P<w_params>(L2-factor|latency|resources|throughput)*\s*\d+)*
                \s*(?P<weight_oc>oc)*
                $""",
                re.VERBOSE,
            ),
            "compval": "cost.dynamic_cost",
            "setval": _tmplt_ip_ospf_cost,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "cost": {
                                "interface_cost": "{{ interface_cost }}",
                                "dynamic_cost": {
                                    "default": "{{ default.split(' ')[1] }}",
                                    "hysteresis": {
                                        "percent": "{{ h_params.split(' ')[1] if h_params is defined and 'percent' in h_params }}",
                                        "threshold": "{{ h_params.split(' ')[1] if h_params is defined and 'threshold' in h_params  }}",
                                    },
                                    "weight": {
                                        "l2_factor": "{{ w_params.split(' ')[1] if w_params is defined and 'L2-factor' in w_params  }}",
                                        "latency": "{{ w_params.split(' ')[1] if w_params is defined and 'latency' in w_params  }}",
                                        "oc": "{{ True if weight_oc is defined}}",
                                        "resources": "{{ w_params.split(' ')[1] if w_params is defined and 'resources' in w_params  }}",
                                        "throughput": "{{ w_params.split(' ')[1] if w_params is defined and 'throughput' in w_params  }}",
                                    },
                                },
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "database_filter",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<database_filter>database-filter\sall\sout)*
                \s*(?P<disable>disable)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} database-filter all out",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "database_filter": "{{ True if database_filter is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "dead_interval",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<dead_interval>dead-interval)*
                \s*(?P<seconds>\d+)*
                \s*(?P<minimal>minimal\shello-multiplier\s\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_dead_interval,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "dead_interval": {
                                "time": "{{ seconds }}",
                                "minimal": {
                                    "hello_multiplier": "{{ minimal.split(' ')[2] }}"
                                },
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "demand_circuit",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<demand_circuit>demand-circuit)*
                \s*(?P<ignore>ignore)*
                \s*(?P<disable>disable)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_demand_circuit,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "demand_circuit": {
                                "enable": "{{ True if demand_circuit is defined and ignore is not defined }}",
                                "ignore": "{{ True if ignore is defined }}",
                                "disable": "{{ True if disable is defined }}",
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "flood_reduction",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<flood_reduction>flood-reduction)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} flood-reduction",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "flood_reduction": "{{ True if flood_reduction is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "hello_interval",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<hello_interval>hello-interval\s\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} hello-interval {{ hello_interval }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "hello_interval": "{{ hello_interval.split(' ')[1] }}",
                        }
                    }
                }
            },
        },
        {
            "name": "lls",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)*
                \s*ospf*
                \s*(?P<lls>lls)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} lls",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "lls": "{{ True if lls is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "manet",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ipv6)*
                \s*ospf*
                \s*(?P<manet>manet\speering)*
                \s*(?P<cost>(cost\spercent|cost\sthreshold)\s\d+)*
                \s*(?P<link_metrics>link-metrics\s\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_manet,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "manet": {
                                "cost": {
                                    "percent": "{{ cost.split(' ')[2] }}",
                                    "threshold": "{{ cost.split(' ')[2] }}",
                                },
                                "link_metrics": {
                                    "set": "{{ True if link_metrics is not defined and link_metrics is defined  }}",
                                    "cost_threshold": "{{ link_metrics.split(' ')[1] }}",
                                },
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "mtu_ignore",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<mtu_ignore>mtu-ignore)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} mtu-ignore",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "mtu_ignore": "{{ True if mtu_ignore is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "multi_area",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)*
                \s*ospf*
                \s*(?P<multi_area>multi-area\s\d+)*
                \s*(?P<cost>cost\s\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_multi_area,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "multi_area": {
                                "id": "{{ multi_area.split(' ')[1] }}",
                                "cost": "{{ cost.split(' ')[1] }}",
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "neighbor",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ipv6)*
                \s*ospf*
                \s*neighbor*
                \s*(?P<address>(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))\S+)*
                \s*(?P<cost>cost\s\d+)*
                \s*(?P<database_filter>database-filter\sall\sout)*
                \s*(?P<poll_interval>poll-interval\s\d+)*
                \s*(?P<priority>priority\s\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_neighbor,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "neighbor": {
                                "address": "{{ address }}",
                                "cost": "{{ cost.split(' ')[1] }}",
                                "database_filter": "{{ True if database_filter is defined }}",
                                "poll_interval": "{{ poll_interval.split(' ')[1] }}",
                                "priority": "{{ priority.split(' ')[1] }}",
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "network",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<network>network)*
                \s*(?P<broadcast>broadcast)*
                \s*(?P<manet>manet)*
                \s*(?P<non_broadcast>non-broadcast)*
                \s*(?P<point_to_multipoint>point-to-multipoint)*
                \s*(?P<point_to_point>point-to-point)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_network,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "network": {
                                "broadcast": "{{ True if broadcast is defined }}",
                                "manet": "{{ True if manet is defined }}",
                                "non_broadcast": "{{ True if non_broadcast is defined }}",
                                "point_to_multipoint": "{{ True if point_to_multipoint is defined }}",
                                "point_to_point": "{{ True if point_to_point is defined }}",
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "prefix_suppression",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<prefix_suppression>prefix-suppression)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} prefix-suppression",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "prefix_suppression": "{{ True if prefix_suppression is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "priority",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<priority>priority*\s*\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} priority {{ priority }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "priority": "{{ priority.split(' ')[1] }}",
                        }
                    }
                }
            },
        },
        {
            "name": "resync_timeout",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)*
                \s*ospf*
                \s*(?P<resync_timeout>resync-timeout*\s*\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": "ip ospf resync-timeout {{ resync_timeout }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "resync_timeout": "{{ resync_timeout.split(' ')[1] }}",
                        }
                    }
                }
            },
        },
        {
            "name": "retransmit_interval",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<retransmit_interval>retransmit-interval*\s*\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} retransmit-interval {{ retransmit_interval }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "retransmit_interval": "{{ retransmit_interval.split(' ')[1] }}",
                        }
                    }
                }
            },
        },
        {
            "name": "shutdown",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)*
                \s*ospf*
                \s*(?P<shutdown>shutdown)*
                $""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ospf' if afi == 'ipv4' else 'ipv6 ospf' }} shutdown",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "shutdown": "{{ True if shutdown is defined }}",
                        }
                    }
                }
            },
        },
        {
            "name": "transmit_delay",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ipv6)*
                \s*ospf*
                \s*(?P<transmit_delay>transmit-delay*\s*\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "ipv6 ospf transmit-delay {{ transmit_delay }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "transmit_delay": "{{ transmit_delay.split(' ')[1] }}",
                        }
                    }
                }
            },
        },
        {
            "name": "ttl_security",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)*
                \s*ospf*
                \s*(?P<ttl_security>ttl-security)*
                \s*(?P<hops>hops\s\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ip_ospf_ttl_security,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "ttl_security": {
                                "set": "{{ True if hops is not defined and ttl_security is defined }}",
                                "hops": "{{ hops.split(' ')[1] }}",
                            },
                        }
                    }
                }
            },
        },
    ]



# (c) 2016, Matt Davis <mdavis@ansible.com>
# (c) 2016, Toshio Kuratomi <tkuratomi@ansible.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import sys
import json

from contextlib import contextmanager
from io import BytesIO, StringIO
from ansible_collections.community.general.tests.unit.compat import unittest
from ansible.module_utils.six import PY3
from ansible.module_utils._text import to_bytes


@contextmanager
def swap_stdin_and_argv(stdin_data='', argv_data=tuple()):
    """
    context manager that temporarily masks the test runner's values for stdin and argv
    """
    real_stdin = sys.stdin
    real_argv = sys.argv

    if PY3:
        fake_stream = StringIO(stdin_data)
        fake_stream.buffer = BytesIO(to_bytes(stdin_data))
    else:
        fake_stream = BytesIO(to_bytes(stdin_data))

    try:
        sys.stdin = fake_stream
        sys.argv = argv_data

        yield
    finally:
        sys.stdin = real_stdin
        sys.argv = real_argv


@contextmanager
def swap_stdout():
    """
    context manager that temporarily replaces stdout for tests that need to verify output
    """
    old_stdout = sys.stdout

    if PY3:
        fake_stream = StringIO()
    else:
        fake_stream = BytesIO()

    try:
        sys.stdout = fake_stream

        yield fake_stream
    finally:
        sys.stdout = old_stdout


class ModuleTestCase(unittest.TestCase):
    def setUp(self, module_args=None):
        if module_args is None:
            module_args = {'_ansible_remote_tmp': '/tmp', '_ansible_keep_remote_files': False}

        args = json.dumps(dict(ANSIBLE_MODULE_ARGS=module_args))

        # unittest doesn't have a clean place to use a context manager, so we have to enter/exit manually
        self.stdin_swap = swap_stdin_and_argv(stdin_data=args)
        self.stdin_swap.__enter__()

    def tearDown(self):
        # unittest doesn't have a clean place to use a context manager, so we have to enter/exit manually
        self.stdin_swap.__exit__(None, None, None)



#!/usr/bin/python
# encoding: utf-8 -*-

# Copyright: (c) 2013, Matthias Vogelgesang <matthias.vogelgesang@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: kernel_blacklist
author:
- Matthias Vogelgesang (@matze)
short_description: Blacklist kernel modules
description:
    - Add or remove kernel modules from blacklist.
options:
    name:
        type: str
        description:
            - Name of kernel module to black- or whitelist.
        required: true
    state:
        type: str
        description:
            - Whether the module should be present in the blacklist or absent.
        choices: [ absent, present ]
        default: present
    blacklist_file:
        type: str
        description:
            - If specified, use this blacklist file instead of
              C(/etc/modprobe.d/blacklist-ansible.conf).
'''

EXAMPLES = '''
- name: Blacklist the nouveau driver module
  community.general.kernel_blacklist:
    name: nouveau
    state: present
'''

import os
import re

from ansible.module_utils.basic import AnsibleModule


class Blacklist(object):
    def __init__(self, module, filename, checkmode):
        self.filename = filename
        self.module = module
        self.checkmode = checkmode

    def create_file(self):
        if not self.checkmode and not os.path.exists(self.filename):
            open(self.filename, 'a').close()
            return True
        elif self.checkmode and not os.path.exists(self.filename):
            self.filename = os.devnull
            return True
        else:
            return False

    def get_pattern(self):
        return r'^blacklist\s*' + self.module + '$'

    def readlines(self):
        f = open(self.filename, 'r')
        lines = f.readlines()
        f.close()
        return lines

    def module_listed(self):
        lines = self.readlines()
        pattern = self.get_pattern()

        for line in lines:
            stripped = line.strip()
            if stripped.startswith('#'):
                continue

            if re.match(pattern, stripped):
                return True

        return False

    def remove_module(self):
        lines = self.readlines()
        pattern = self.get_pattern()

        if self.checkmode:
            f = open(os.devnull, 'w')
        else:
            f = open(self.filename, 'w')

        for line in lines:
            if not re.match(pattern, line.strip()):
                f.write(line)

        f.close()

    def add_module(self):
        if self.checkmode:
            f = open(os.devnull, 'a')
        else:
            f = open(self.filename, 'a')

        f.write('blacklist %s\n' % self.module)

        f.close()


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['absent', 'present']),
            blacklist_file=dict(type='str')
        ),
        supports_check_mode=True,
    )

    args = dict(changed=False, failed=False,
                name=module.params['name'], state=module.params['state'])

    filename = '/etc/modprobe.d/blacklist-ansible.conf'

    if module.params['blacklist_file']:
        filename = module.params['blacklist_file']

    blacklist = Blacklist(args['name'], filename, module.check_mode)

    if blacklist.create_file():
        args['changed'] = True
    else:
        args['changed'] = False

    if blacklist.module_listed():
        if args['state'] == 'absent':
            blacklist.remove_module()
            args['changed'] = True
    else:
        if args['state'] == 'present':
            blacklist.add_module()
            args['changed'] = True

    module.exit_json(**args)


if __name__ == '__main__':
    main()
